import './styles/reset.less';
import 'bootstrap/dist/css/bootstrap.min.css';
import 'xl-icons/style.less';

import React from 'react';
import {render} from 'react-dom';
import {AppContainer} from 'react-hot-loader';
import {Provider} from 'react-redux';
import {Redirect, Switch, Route} from 'react-router';
import {ConnectedRouter} from 'connected-react-router';
import {App} from './pages/App';
import {Buttons} from './pages/buttons-page/Buttons';
import {Dip} from './pages/dip-page/Dip';
import {Form} from './pages/form-page/Form';
import {Home} from './pages/Home';
import {Tabs} from './pages/tabs-page/Tabs';
import {Widget} from './pages/widget-page/Widget';
import {store} from './pages/store/plumbing';
import history from './history';

import './kitchen-sink/styles/kitchen-sink.less';

const redirectHome = () => <Redirect to="/home"/>;

const renderApp = (ComponentsApp) => {
    render(
        <AppContainer>
            <Provider store={store}>
                <ConnectedRouter history={history}>
                    <div>
                        <Route exact={true} path="/" render={redirectHome}/>
                        <ComponentsApp>
                            <Switch>
                                <Route component={Buttons} exact={true} path="/buttons"/>
                                <Route component={Buttons} exact={true} path="/button/:name"/>
                                <Route component={Tabs} exact={true} path="/tabs"/>
                                <Route component={Form} exact={true} path="/form"/>
                                <Route component={Form} exact={true} path="/form/:name"/>
                                <Route component={Dip} exact={true} path="/dip"/>
                                <Route component={Dip} exact={true} path="/dip/:name"/>
                                <Route component={Widget} exact={true} path="/widget"/>
                                <Route component={Widget} exact={true} path="/widget/:name"/>
                                <Route component={Home} path="/home"/>
                            </Switch>
                        </ComponentsApp>
                    </div>
                </ConnectedRouter>
            </Provider>
        </AppContainer>,
        document.getElementById('root')
    );
};

renderApp(App);

if (module.hot) {
    module.hot.accept(() => {
        renderApp(App);
    });
}
