import R from 'ramda';
import React, {Component} from 'react';
import {connect} from 'react-redux';
import PropTypes from 'prop-types';
import classNames from 'classnames';
import {ComponentNavigator} from '../playground/component-navigator/component-navigator';
import DipKitchenTestComponent from '../playground/dip-kitchen-test-component';
import {XlReactDipInteger} from '../../xl-react-dip/xl-react-dip-integer/xl-react-dip-integer';
import dipActions from '../actions/dip-actions';
import {XlReactDipList} from '../../xl-react-dip/xl-react-dip-list/xl-react-dip-list';
import {XlReactDipBoolean} from '../../xl-react-dip/xl-react-dip-boolean/xl-react-dip-boolean';
import {XlReactDipCi} from '../../xl-react-dip/xl-react-dip-ci/xl-react-dip-ci';
import {XlReactDipEnum} from '../../xl-react-dip/xl-react-dip-enum/xl-react-dip-enum';
import {XlReactDipListOfCi} from '../../xl-react-dip/xl-react-dip-list-of-ci/xl-react-dip-list-of-ci';
import {XlReactDipMapStringString} from
    '../../xl-react-dip/xl-react-dip-map-string-string/xl-react-dip-map-string-string';
import {XlReactDipSetOfCi} from '../../xl-react-dip/xl-react-dip-set-of-ci/xl-react-dip-set-of-ci';
import {XlReactDipString} from '../../xl-react-dip/xl-react-dip-string/xl-react-dip-string';
import {XlReactDipSetOfStrings} from '../../xl-react-dip/xl-react-dip-set-of-strings/xl-react-dip-set-of-strings';
import {XlReactDipDatetime} from '../../xl-react-dip/xl-react-dip-datetime/xl-react-dip-datetime';
import {getInitialContext} from '../../xl-react-form/xl-react-dip-dynamic-form-service';
import {WARNING} from '../../utils/ci-utils';
import {autocompleteHandlers, getHandlers} from '../handlers/predefined-handlers';
import {getCollectionOfCiHandlers} from './dip-utils';

import './dip-page.less';

const mapStateToProps = (state) => ({dipContext: state.dip});
const mapDispatchToProps = {updateModelState: dipActions.updateModelState};

@connect(mapStateToProps, mapDispatchToProps)
export class Dip extends Component {
    static propTypes = {
        dipContext: PropTypes.object,
        match: PropTypes.object,
        updateModelState: PropTypes.func
    };

    constructor(props) {
        super(props);
        const handlers = {
            CI: {autocompleteHandlers},
            LIST_OF_CI: getCollectionOfCiHandlers,
            LIST_OF_STRING: {
                NameList3: getCollectionOfCiHandlers,
                NameList4: getHandlers()
            },
            SET_OF_CI: getCollectionOfCiHandlers
        };
        R.pipe(
            R.keys,
            R.forEach((key) => {
                const componentProps = R.path(['dipContext', key, 'componentProps'], props);
                const propMetadata = R.prop('metadata', componentProps);
                const name = R.prop('name', propMetadata);
                const model = R.propOr(R.prop('default', propMetadata), 'model', componentProps);
                const contextPath = ['componentProps', 'context'];
                const initialContext = getInitialContext(
                    {contextPath, handlers, metadata: {properties: [propMetadata]}},
                    {[name]: model}
                );
                const context = R.prop(name, initialContext);
                props.updateModelState(key, {
                    componentProps: {
                        ...componentProps,
                        context,
                        model: context.transformModel(model),
                        pristine: true,
                        valid: true
                    }
                });
            })
        )(props.dipContext);
    }

    render() {
        const {dipContext, match} = this.props;

        const updateState = (modelName, newState) => {
            this.props.updateModelState(modelName, newState);
        };

        const mergePropState = (modelName) => (state) => {
            updateState(modelName, {state});
        };

        const warningValidator = () => ({
            condition: true,
            message: {show: false, text: 'Sample Warning', type: WARNING}
        });

        const dips = {
            dipArrayOfStrings1: <DipKitchenTestComponent
                className="xl-react-dip-list-without-auto-complete"
                currentState={dipContext}
                key={'dipArrayOfStrings1'}
                label={'Without autocomplete handler'}
                modelName={'dipArrayOfStrings1'}
                updateState={updateState}>
                <XlReactDipList
                    handlers={{NameList3: getCollectionOfCiHandlers}}
                    mergeContext={mergePropState('dipArrayOfStrings1')}
                />
            </DipKitchenTestComponent>,
            dipArrayOfStrings2: <DipKitchenTestComponent
                className="xl-react-dip-list-with-auto-complete"
                currentState={dipContext}
                key={'dipArrayOfStrings2'}
                label={'With autocomplete handler'}
                modelName={'dipArrayOfStrings2'}
                updateState={updateState}>
                <XlReactDipList
                    handlers={{NameList4: getHandlers()}}
                    mergeContext={mergePropState('dipArrayOfStrings2')}
                />
            </DipKitchenTestComponent>,
            dipBoolean1: <DipKitchenTestComponent
                className="xl-react-dip-boolean"
                currentState={dipContext}
                key={'dipBoolean1'}
                modelName={'dipBoolean1'}
                updateState={updateState}>
                <XlReactDipBoolean/>
            </DipKitchenTestComponent>,
            dipCi1: <DipKitchenTestComponent
                className="xl-react-dip-ci-initial"
                currentState={dipContext}
                key={'dipCi1'}
                label={'CI'}
                modelName={'dipCi1'}
                updateState={updateState}>
                <XlReactDipCi handlers={autocompleteHandlers} objLabel="id"/>
            </DipKitchenTestComponent>,
            dipCi2: <DipKitchenTestComponent
                className="xl-react-dip-ci-with-default-value"
                currentState={dipContext}
                key={'dipCi2'}
                label={'CI with given value'}
                modelName={'dipCi2'}
                updateState={updateState}>
                <XlReactDipCi handlers={autocompleteHandlers} objLabel="id"/>
            </DipKitchenTestComponent>,
            dipDateTime1: <DipKitchenTestComponent
                className={'xl-react-dip-datetime-component'}
                currentState={dipContext}
                key={'dipDateTime1'}
                label={'Date and time'}
                modelName={'dipDateTime1'}
                updateState={updateState}>
                <XlReactDipDatetime/>
            </DipKitchenTestComponent>,
            dipEnum1: <DipKitchenTestComponent
                className="xl-react-dip-enum-initial"
                currentState={dipContext}
                key={'dipEnum1'}
                label={'Enumeration of strings'}
                modelName={'dipEnum1'}
                updateState={updateState}>
                <XlReactDipEnum/>
            </DipKitchenTestComponent>,
            dipInteger1: <DipKitchenTestComponent
                className="xl-react-dip-integer-default"
                currentState={dipContext}
                key={'dipInteger1'}
                label={'Without default value'}
                modelName={'dipInteger1'}
                updateState={updateState}
                validators={[warningValidator]}>
                <XlReactDipInteger/>
            </DipKitchenTestComponent>,
            dipInteger2: <DipKitchenTestComponent
                className="xl-react-dip-integer-with-default-value"
                currentState={dipContext}
                key={'dipInteger2'}
                label={'With default value'}
                modelName={'dipInteger2'}
                updateState={updateState}>
                <XlReactDipInteger/>
            </DipKitchenTestComponent>,
            dipListOfCi1: <DipKitchenTestComponent
                className={'dip-list-of-ci-component'}
                currentState={dipContext}
                key={'dipListOfCi1'}
                label={'List Of CI'}
                modelName={'dipListOfCi1'}
                updateState={updateState}>
                <XlReactDipListOfCi
                    emptyPlaceholder={dipContext.dipListOfCi1.emptyPlaceholder}
                    handlers={getCollectionOfCiHandlers}
                    newItemSuffix={dipContext.dipListOfCi1.newItemSuffix}
                    objLabel={'id'}
                    openOnClick={false}
                />
            </DipKitchenTestComponent>,
            dipMapStringString1: <DipKitchenTestComponent
                className="xl-react-dip-map-string-string-initial"
                currentState={dipContext}
                key={'dipMapStringString1'}
                label={'Map string of strings'}
                modelName={'dipMapStringString1'}
                updateState={updateState}>
                <XlReactDipMapStringString/>
            </DipKitchenTestComponent>,
            dipSetOfCi1: <DipKitchenTestComponent
                className={'dip-set-of-ci-component'}
                currentState={dipContext}
                key={'dipSetOfCi1'}
                label={'Set Of CI'}
                modelName={'dipSetOfCi1'}
                updateState={updateState}>
                <XlReactDipSetOfCi
                    emptyPlaceholder={dipContext.dipSetOfCi1.emptyPlaceholder}
                    handlers={getCollectionOfCiHandlers}
                    newItemSuffix={dipContext.dipSetOfCi1.newItemSuffix}
                    objLabel={'id'}
                    openOnClick={false}
                />
            </DipKitchenTestComponent>,
            dipString1: <DipKitchenTestComponent
                className="xl-react-dip-string"
                currentState={dipContext}
                key={'dipString1'}
                label={'Without default value'}
                modelName={'dipString1'}
                updateState={updateState}>
                <XlReactDipString/>
            </DipKitchenTestComponent>,
            dipString2: <DipKitchenTestComponent
                className="xl-react-dip-string-with-default-value"
                currentState={dipContext}
                key={'dipString2'}
                label={'With default value'}
                modelName={'dipString2'}
                updateState={updateState}>
                <XlReactDipString/>
            </DipKitchenTestComponent>,
            dipStringSet1: <DipKitchenTestComponent
                className={'dip-set-of-strings-component'}
                currentState={dipContext}
                key={'dipStringSet1'}
                label={'Set of strings'}
                modelName={'dipStringSet1'}
                updateState={updateState}>
                <XlReactDipSetOfStrings/>
            </DipKitchenTestComponent>
        };

        const isSingleItem = match && dips[match.params.name];

        return (
            <div className={classNames('dip-page', {'single-item': isSingleItem})}>
                <ComponentNavigator componentType="dip" state={dipContext}/>
                <div className="page-content">
                    {isSingleItem ? dips[match.params.name] : R.values(dips)}
                </div>
            </div>
        );
    }
}
