/* eslint-disable sort-keys, max-lines */
import moment from 'moment';
import {enumViews} from '../../xl-react-dip/xl-react-dip-enum/dip-enum-service';
import {bindingTypes} from '../playground/bindings/component-bindings';
import {commonBindings, enumInputTypeBinding, inputChangeViewBinding} from '../playground/bindings/initial-bindings';

const type = 'dip';

export const dipInitialState = {
    dipBoolean1: {
        navigation: {
            description: 'xl-react-dip-boolean',
            title: 'DIP Boolean',
            type
        },
        componentProps: {
            displayMode: 'edit',
            model: false,
            metadata: {
                kind: 'BOOLEAN',
                label: 'I\'m a boolean',
                description: 'This is my check field.'
            }
        },
        bindings: commonBindings('edit')
    },
    dipInteger1: {
        navigation: {
            description: 'xl-react-dip-integer',
            title: 'XL DIP Integer',
            type
        },
        componentProps: {
            displayMode: 'edit',
            metadata: {
                kind: 'INTEGER',
                label: 'I\'m an integer',
                description: 'This is my int field.'
            }
        },
        bindings: commonBindings()
    },
    dipInteger2: {
        navigation: {
            description: 'xl-react-dip-integer',
            title: 'XL DIP Integer Default',
            type
        },
        componentProps: {
            displayMode: 'edit',
            metadata: {
                default: 30,
                kind: 'INTEGER',
                label: 'I\'m an integer',
                description: 'This is my int field.'
            }
        },
        bindings: commonBindings('edit')
    },
    dipString1: {
        navigation: {
            description: 'xl-react-dip-string',
            title: 'XL Dip String',
            type
        },
        componentProps: {
            metadata: {
                kind: 'STRING',
                label: 'String me',
                name: 'application.string',
                description: 'This is my string field.',
                placeholder: 'String placeholder'
            }
        },
        bindings: [
            ...commonBindings(),
            enumInputTypeBinding
        ]
    },
    dipString2: {
        navigation: {
            description: 'xl-react-dip-string',
            title: 'XL Dip String Default',
            type
        },
        componentProps: {
            metadata: {
                default: 'value',
                kind: 'STRING',
                label: 'String me',
                name: 'application.string',
                description: 'This is my string field.'
            }
        },
        bindings: [...commonBindings()]
    },
    dipCi1: {
        navigation: {
            description: 'xl-react-dip-ci',
            title: 'XL DIP Ci',
            type
        },
        componentProps: {
            metadata: {
                description: 'This is my autocomplete field.',
                kind: 'CI',
                label: 'Autocomplete',
                name: 'application.autocomplete'
            },
            pristine: true,
            valid: true
        },
        bindings: commonBindings()
    },
    dipCi2: {
        navigation: {
            description: 'xl-react-dip-ci',
            title: 'XL DIP Ci Default',
            type
        },
        componentProps: {
            metadata: {
                description: 'With simple string',
                default: 'Application2',
                kind: 'CI',
                label: 'Ci',
                name: 'application.autocomplete',
                referencedType: 'referencedType'
            }
        },
        bindings: commonBindings()
    },
    dipArrayOfStrings1: {
        navigation: {
            description: 'xl-react-dip-list',
            title: 'DIP List',
            type
        },
        componentProps: {
            metadata: {
                default: [
                    'Item 1',
                    'Item 2',
                    'Item 3',
                    'Item 4'
                ],
                kind: 'LIST_OF_STRING',
                name: 'NameList3',
                label: 'Array of strings',
                description: 'Find the description here',
                emptyPlaceholder: 'No items',
                requiredText: 'Custom required text overriding default required text!'
            }
        },
        bindings: commonBindings()
    },
    dipArrayOfStrings2: {
        navigation: {
            description: 'xl-react-dip-list',
            title: 'XL DIP List With Autocomplete',
            type
        },
        componentProps: {
            metadata: {
                default: [
                    'Application1',
                    'Application4'
                ],
                kind: 'LIST_OF_STRING',
                name: 'NameList4',
                label: 'Array of strings with suggestions',
                description: 'Can provide you some suggestions',
                emptyPlaceholder: 'No items yet'
            }
        },
        bindings: commonBindings()
    },
    dipEnum1: {
        navigation: {
            description: 'xl-react-dip-enum',
            title: 'DIP Enum',
            type
        },
        componentProps: {
            metadata: {
                enumValues: [
                    'ace',
                    'albatross',
                    'eagle',
                    'birdie',
                    'par',
                    'bogey'
                ],
                kind: 'ENUM',
                label: 'Enum Label',
                default: 'birdie',
                description: 'Select a string',
                name: 'dipEnum'
            },
            model: 'eagle',
            name: 'dipEnum'
        },
        bindings: [
            ...commonBindings(),
            inputChangeViewBinding(enumViews)
        ]
    },
    dipMapStringString1: {
        navigation: {
            description: 'xl-react-dip-map-string-string',
            title: 'DIP Map String String',
            type
        },
        componentProps: {
            metadata: {
                description: 'Find the description here',
                kind: 'MAP_STRING_STRING',
                label: 'Dictionary',
                name: 'NameMap'
            },
            model: {
                'Name': 'Marc',
                'Surname': 'Dupont',
                'Age': 39,
                'Street': '433 Boulevard des templiers',
                'Zipcode': 74003,
                'City': 'Paris',
                'Phone': '0123456789',
                'Mobile Phone': '+31123456789'
            },
            pristine: true,
            valid: true
        },
        bindings: [
            ...commonBindings(),
            {
                className: 'password',
                label: 'Password',
                name: 'metadata.password',
                type: bindingTypes.BOOLEAN,
                value: false
            }
        ]
    },
    dipDateTime1: {
        navigation: {
            description: 'xl-react-dip-datetime',
            title: 'DIP Date time',
            type
        },
        componentProps: {
            model: moment({
                year: 2016,
                month: 10,
                day: 9,
                hours: 15,
                minutes: 10,
                seconds: 0,
                milliseconds: 0
            }).toDate(),
            metadata: {
                kind: 'DATE',
                label: 'Datetime Label',
                description: 'Pick a date and time'
            }
        },
        bindings: commonBindings()
    },
    dipStringSet1: {
        navigation: {
            description: 'dip-set-of-strings',
            title: 'DIP Set Of Strings',
            type
        },
        componentProps: {
            metadata: {
                description: 'Find the description here',
                kind: 'SET_OF_STRING',
                label: 'My items',
                name: 'NameSet3',
                requiredText: 'Custom required text overriding default required text!'
            },
            model: ['Item 1', 'Item 2', 'Item 3', 'Item 4'],
            pristine: true,
            valid: true
        },
        bindings: [...commonBindings()]
    },
    dipListOfCi1: {
        navigation: {
            description: 'dip-list-of-ci',
            title: 'DIP List Of CI',
            type
        },
        componentProps: {
            emptyPlaceholder: 'No items selected yet',
            metadata: {
                kind: 'LIST_OF_CI',
                name: 'NameListOfCi1',
                label: 'List of CIs',
                description: 'Find the description here'
            },
            model: ['Item 1', 'Item 2', 'Item 3', 'Item 4', 'Item 5'],
            newItemSuffix: ' (New CI)'
        },
        bindings: commonBindings()
    },
    dipSetOfCi1: {
        navigation: {
            description: 'dip-set-of-ci',
            title: 'DIP Set Of CI',
            type
        },
        componentProps: {
            emptyPlaceholder: 'No items selected yet',
            metadata: {
                kind: 'SET_OF_CI',
                name: 'NameSetOfCi1',
                label: 'Set of CIs',
                description: 'Find the description here'
            },
            model: ['Item 1', 'Item 5', 'Item 4', 'Item 2', 'Item 3'],
            newItemSuffix: ' (New CI)'
        },
        bindings: [...commonBindings()]
    }
};
/* eslint-enable sort-keys, max-lines */
