/* eslint-disable sort-keys, object-curly-newline, no-multi-str, max-lines, quotes, quote-props */
import R from 'ramda';
import RU from '../../utils/ramda-utils';
import {commonBindings} from '../playground/bindings/initial-bindings';
import {vaultValidator} from './form-validators';

const type = 'form';
const bindings = commonBindings('creation', null, 'pristinity', 'validity');

const dynamicFormDefault = {
    formId: 'dynamicFormDefault',
    model: {},
    pristinity: {},
    validity: {},
    bindings,
    metadata: {
        type: "jea.TopicConnectionFactorySpec",
        description: "Description unavailable",
        properties: [
            {
                default: "component-0",
                description: "Description component String",
                inputHint: {
                    rules: [
                        {
                            pattern: '^comp',
                            message: 'Value should start with comp'
                        },
                        {
                            pattern: '0$',
                            message: 'Value should end on 0'
                        }
                    ]
                },
                label: "Label component String",
                kind: "STRING",
                name: "nameComponentString",
                required: false,
                password: false,
                transient: false,
                size: "DEFAULT"
            },
            {
                name: "nameComponentStringWithTypeHint",
                label: "Label component StringWithTypeHint",
                kind: "STRING",
                description: "Description component String",
                required: false,
                password: false,
                transient: false,
                size: "DEFAULT",
                default: "string",
                inputHint: {
                    kind: 'ENUM',
                    values: [
                        {value: 'eu-west-1', label: 'EU (Ireland)'},
                        {value: 'eu-west-2', label: 'EU (London)'}
                    ]
                }
            },
            {
                name: "nameComponentStringLarge",
                label: "Label component StringLarge",
                kind: "STRING",
                description: "Description component StringLarge",
                required: false,
                password: false,
                transient: false,
                size: "LARGE",
                default: "large string"
            },
            {
                name: "nameComponentStringMonaco",
                label: "Label component StringMonaco",
                kind: "STRING",
                description: "Description component StringMonaco",
                required: false,
                password: false,
                transient: false,
                view: "monaco",
                default: "monaco string"
            },
            {
                name: "nameComponentInteger",
                label: "Label component Integer",
                kind: "INTEGER",
                description: "Description component Integer",
                required: false,
                password: false,
                transient: false,
                size: "DEFAULT",
                default: "22"
            },
            {
                name: "nameComponentBoolean",
                label: "Label component Boolean",
                kind: "BOOLEAN",
                description: "Description component Boolean",
                required: false,
                password: false,
                transient: false,
                size: "DEFAULT",
                default: true
            },
            {
                name: "nameComponentSet",
                label: "Label component Set",
                kind: "SET_OF_STRING",
                description: "Description component Set",
                required: false,
                password: false,
                size: "DEFAULT",
                default: ['apple', 'grape']
            },
            {
                name: "nameComponentList",
                label: "Label component List",
                kind: "LIST_OF_STRING",
                description: "Description component List",
                required: false,
                password: false,
                size: "DEFAULT",
                default: [
                    "tangerine",
                    "mango"
                ]
            },
            {
                name: "nameComponentCi",
                label: "Label component CI",
                kind: "CI",
                description: "Description component CI",
                required: false,
                password: false,
                size: "DEFAULT",
                default: "CiName",
                "referencedType": "ciType"
            },
            {
                name: "nameStringEnum",
                label: "Label String enum",
                kind: "ENUM",
                description: "Description enum of string",
                required: false,
                password: false,
                size: "DEFAULT",
                default: "mario",
                "enumValues": [
                    "mario",
                    "luigi",
                    "bowser",
                    "wario",
                    "toad",
                    "yoshi"
                ]
            },
            {
                default: 'mario',
                name: "nameRadioEnum",
                label: "Label Radio enum",
                kind: "ENUM",
                description: "Description enum of strings as radio",
                required: false,
                password: false,
                size: "DEFAULT",
                enumValues: [
                    "mario",
                    "luigi"
                ],
                view: 'radio'
            },
            {
                name: "mapStringStringName",
                label: "Label component Map String String",
                kind: "MAP_STRING_STRING",
                description: "Description component Map String String",
                required: false,
                password: false,
                transient: false,
                default: {
                    "someKey": "someValue"
                }
            },
            {
                name: "nameSetOfCi",
                label: "Label component Set Of CI",
                kind: "SET_OF_CI",
                description: "Description component Set Of CI",
                required: false,
                password: false,
                transient: false,
                default: [
                    {
                        id: "Infrastructure/TomcatHost",
                        type: "CI"
                    },
                    {
                        id: "Applications/TestApp/1.0",
                        type: "CI"
                    },
                    {
                        id: "Applications/ZooApp/2.0",
                        type: "CI"
                    },
                    {
                        id: "Applications/ZooApp/1.0",
                        type: "CI"
                    }
                ]
            },
            {
                name: "nameListOfCi",
                label: "Label component List Of CI",
                kind: "LIST_OF_CI",
                description: "Description component List Of CI",
                required: false,
                password: false,
                transient: false,
                default: [
                    {
                        id: "List Item 1",
                        type: "10"
                    },
                    {
                        id: "Application1",
                        type: "10"
                    }
                ]
            },
            {
                name: "datetimeName",
                label: "Label component Date Time",
                kind: "DATE",
                description: "Description component Date Time",
                required: false,
                password: false,
                transient: false,
                default: '2016-11-09T15:10:00.000'
            },
            {
                name: "datetimeStringName",
                label: "Label component Date Time With String Input",
                kind: "DATE",
                description: "Description component Date Time",
                required: false,
                password: false,
                transient: false,
                default: '2017-11-09T15:10:00.000'
            }
        ],
        interfaces: [
            "udm.SourceArtifact",
            "udm.Plop"
        ]
    },
    navigation: {
        description: 'xl-react-dip-dynamic-form',
        title: 'DIP Dynamic Form (Default)',
        type
    }
};

export const formInitialState = {
    dynamicFormDefault,
    dynamicFormEmpty: {
        formId: 'dynamicFormEmpty',
        model: {},
        pristinity: {},
        validity: {},
        bindings,
        metadata: {
            type: "jea.TopicConnectionFactorySpec",
            description: "Description unavailable",
            properties: [
                {
                    name: "nameComponentString",
                    label: "Label component String",
                    kind: "STRING",
                    description: "Description component String",
                    required: false,
                    password: false,
                    transient: false,
                    size: "DEFAULT"
                },
                {
                    name: "nameComponentStringWithTypeHint",
                    label: "Label component StringWithTypeHint",
                    kind: "STRING",
                    description: "Description component String",
                    required: false,
                    password: false,
                    transient: false,
                    size: "DEFAULT",
                    inputHint: {
                        kind: 'ENUM',
                        values: [
                            {value: 'eu-west-1', label: 'EU (Ireland)'},
                            {value: 'eu-west-2', label: 'EU (London)'}
                        ]
                    }
                },
                {
                    name: "nameComponentStringLarge",
                    label: "Label component StringLarge",
                    kind: "STRING",
                    description: "Description component StringLarge",
                    required: false,
                    password: false,
                    transient: false,
                    size: "LARGE"
                },
                {
                    name: "nameComponentStringMonaco",
                    label: "Label component StringMonaco",
                    kind: "STRING",
                    description: "Description component StringMonaco",
                    required: false,
                    password: false,
                    transient: false,
                    view: "monaco"
                },
                {
                    name: "nameComponentInteger",
                    label: "Label component Integer",
                    kind: "INTEGER",
                    description: "Description component Integer",
                    required: false,
                    password: false,
                    transient: false,
                    size: "DEFAULT"
                },
                {
                    name: "nameComponentBoolean",
                    label: "Label component Boolean",
                    kind: "BOOLEAN",
                    description: "Description component Boolean",
                    required: false,
                    password: false,
                    transient: false,
                    size: "DEFAULT"
                },
                {
                    name: "nameComponentSet",
                    label: "Label component Set",
                    kind: "SET_OF_STRING",
                    description: "Description component Set",
                    required: false,
                    password: false,
                    size: "DEFAULT"
                },
                {
                    name: "nameComponentList",
                    label: "Label component List",
                    kind: "LIST_OF_STRING",
                    description: "Description component List",
                    required: false,
                    password: false,
                    size: "DEFAULT"
                },
                {
                    name: "nameComponentCi",
                    label: "Label component CI",
                    kind: "CI",
                    description: "Description component CI",
                    required: false,
                    password: false,
                    size: "DEFAULT",
                    "referencedType": "ciType"
                },
                {
                    name: "nameStringEnum",
                    label: "Label String enum",
                    kind: "ENUM",
                    description: "Description enum of string",
                    required: false,
                    password: false,
                    size: "DEFAULT",
                    "enumValues": [
                        "mario",
                        "luigi",
                        "bowser",
                        "wario",
                        "toad",
                        "yoshi"
                    ]
                },
                {
                    name: "nameRadioEnum",
                    label: "Label Radio enum",
                    kind: "ENUM",
                    description: "Description enum of strings as radio",
                    required: false,
                    password: false,
                    size: "DEFAULT",
                    enumValues: [
                        "mario",
                        "luigi"
                    ],
                    view: 'radio'
                },
                {
                    name: "mapStringStringName",
                    label: "Label component Map String String",
                    kind: "MAP_STRING_STRING",
                    description: "Description component Map String String",
                    required: false,
                    password: false,
                    transient: false
                },
                {
                    name: "nameSetOfCi",
                    label: "Label component Set Of CI",
                    kind: "SET_OF_CI",
                    description: "Description component Set Of CI",
                    required: false,
                    password: false,
                    transient: false
                },
                {
                    name: "nameListOfCi",
                    label: "Label component List Of CI",
                    kind: "LIST_OF_CI",
                    description: "Description component List Of CI",
                    required: false,
                    password: false,
                    transient: false
                },
                {
                    name: "datetimeName",
                    label: "Label component Date Time",
                    kind: "DATE",
                    description: "Description component Date Time",
                    required: false,
                    password: false,
                    transient: false,
                    default: '2016-11-09T15:10:00.000'
                },
                {
                    name: "datetimeStringName",
                    label: "Label component Date Time With String Input",
                    kind: "DATE",
                    description: "Description component Date Time",
                    required: false,
                    password: false,
                    transient: false,
                    default: '2017-11-09T15:10:00.000'
                }
            ],
            interfaces: [
                "udm.SourceArtifact",
                "udm.Plop"
            ]
        },
        extraBindingSections: [
            {
                type: 'select',
                label: 'Focus The Field',
                name: 'focusOnField',
                value: '',
                onChangeCallApi: (api, value) => {
                    RU.invoke('focusFieldByName', [value], api);
                },
                values: [
                    'nameComponentString',
                    'nameComponentStringLarge',
                    'nameComponentStringMonaco',
                    'nameComponentInteger',
                    'nameComponentSet',
                    'nameComponentList',
                    'nameComponentCi',
                    'nameStringEnum',
                    'mapStringStringName',
                    'nameSetOfCi',
                    'nameListOfCi',
                    'datetimeName'
                ]
            }
        ],
        navigation: {
            description: 'xl-react-dip-dynamic-form',
            title: 'DIP Dynamic Form (Empty)',
            type
        }
    },
    dynamicFormEmptyWithWarnings: {
        formId: 'dynamicFormEmptyWithWarnings',
        model: {},
        pristinity: {},
        validity: {},
        bindings,
        metadata: {
            type: "jea.TopicConnectionFactorySpec",
            description: "Description unavailable",
            properties: [
                {
                    name: "nameComponentString",
                    label: "Label component String",
                    kind: "STRING",
                    description: "Description component String",
                    required: false,
                    password: false,
                    transient: false,
                    size: "DEFAULT",
                    inputHint: {
                        rules: [
                            {
                                pattern: '^comp',
                                message: 'Value should start with comp'
                            },
                            {
                                pattern: '0$',
                                message: 'Value should end on 0'
                            }
                        ]
                    }
                },
                {
                    name: "nameComponentStringWithTypeHint",
                    label: "Label component StringWithTypeHint",
                    kind: "STRING",
                    description: "Description component String",
                    required: false,
                    password: false,
                    transient: false,
                    size: "DEFAULT",
                    inputHint: {
                        kind: 'ENUM',
                        rules: [
                            {
                                pattern: 'str',
                                message: 'Value should contain `str` in the value'
                            },
                            {
                                pattern: '^(?:(?!string).)*$',
                                message: 'Value should not contain `string` in the value'
                            }
                        ],
                        values: [
                            {value: 'eu-west-1', label: 'EU (Ireland)'},
                            {value: 'eu-west-2', label: 'EU (London)'}
                        ]
                    }
                },
                {
                    name: "nameComponentStringLarge",
                    label: "Label component StringLarge",
                    kind: "STRING",
                    description: "Description component StringLarge",
                    required: false,
                    password: false,
                    transient: false,
                    size: "LARGE",
                    inputHint: {
                        rules: [
                            {
                                pattern: 'str',
                                message: 'Value should contain `str` in the value'
                            },
                            {
                                pattern: '^(?:(?!string).)*$',
                                message: 'Value should not contain `string` in the value'
                            }
                        ]
                    }
                },
                {
                    name: "nameComponentStringMonaco",
                    label: "Label component StringMonaco",
                    kind: "STRING",
                    description: "Description component StringMonaco",
                    required: false,
                    password: false,
                    transient: false,
                    view: "monaco",
                    inputHint: {
                        rules: [
                            {
                                pattern: 'monaco editor',
                                message: 'Value should contain `monaco editor` in the value'
                            }
                        ]
                    }
                },
                {
                    name: "nameComponentInteger",
                    label: "Label component Integer",
                    kind: "INTEGER",
                    description: "Description component Integer",
                    required: false,
                    password: false,
                    transient: false,
                    size: "DEFAULT",
                    inputHint: {
                        rules: [
                            {
                                pattern: '^[0-5]+$',
                                message: 'Value should contain digits between 0 and 5 inclusively'
                            }
                        ]
                    }
                },
                {
                    name: "nameComponentBoolean",
                    label: "Label component Boolean",
                    kind: "BOOLEAN",
                    description: "Description component Boolean",
                    required: false,
                    password: false,
                    transient: false,
                    size: "DEFAULT"
                },
                {
                    name: "nameComponentSet",
                    label: "Label component Set",
                    kind: "SET_OF_STRING",
                    description: "Description component Set",
                    required: false,
                    password: false,
                    size: "DEFAULT"
                },
                {
                    name: "nameComponentList",
                    label: "Label component List",
                    kind: "LIST_OF_STRING",
                    description: "Description component List",
                    required: false,
                    password: false,
                    size: "DEFAULT"
                },
                {
                    name: "nameComponentCi",
                    label: "Label component CI",
                    kind: "CI",
                    description: "Description component CI",
                    required: false,
                    password: false,
                    size: "DEFAULT",
                    "referencedType": "ciType",
                    inputHint: {
                        rules: [
                            {
                                pattern: '^App',
                                message: 'Value should start with App'
                            }
                        ]
                    }
                },
                {
                    name: "nameStringEnum",
                    label: "Label String enum",
                    kind: "ENUM",
                    description: "Description enum of string",
                    required: false,
                    password: false,
                    size: "DEFAULT",
                    "enumValues": [
                        "mario",
                        "luigi",
                        "bowser",
                        "wario",
                        "toad",
                        "yoshi"
                    ],
                    inputHint: {
                        rules: [
                            {
                                pattern: '^.{1,5}$',
                                message: 'Value should be less than 5 symbols'
                            }
                        ]
                    }
                },
                {
                    name: "nameRadioEnum",
                    label: "Label Radio enum",
                    kind: "ENUM",
                    description: "Description enum of strings as radio",
                    required: false,
                    password: false,
                    size: "DEFAULT",
                    enumValues: [
                        "mario",
                        "luigi",
                        "bowser",
                        "wario",
                        "toad",
                        "yoshi"
                    ],
                    view: 'radio',
                    inputHint: {
                        rules: [
                            {
                                pattern: '^.{1,5}$',
                                message: 'Value should be less than 5 symbols'
                            }
                        ]
                    }
                },
                {
                    name: "mapStringStringName",
                    label: "Label component Map String String",
                    kind: "MAP_STRING_STRING",
                    description: "Description component Map String String",
                    required: false,
                    password: false,
                    transient: false
                },
                {
                    name: "nameSetOfCi",
                    label: "Label component Set Of CI",
                    kind: "SET_OF_CI",
                    description: "Description component Set Of CI",
                    required: false,
                    password: false,
                    transient: false
                },
                {
                    name: "nameListOfCi",
                    label: "Label component List Of CI",
                    kind: "LIST_OF_CI",
                    description: "Description component List Of CI",
                    required: false,
                    password: false,
                    transient: false
                },
                {
                    name: "datetimeName",
                    label: "Label component Date Time",
                    kind: "DATE",
                    description: "Description component Date Time",
                    required: false,
                    password: false,
                    transient: false
                },
                {
                    name: "datetimeStringName",
                    label: "Label component Date Time With String Input",
                    kind: "DATE",
                    description: "Description component Date Time",
                    required: false,
                    password: false,
                    transient: false,
                    default: '2017-11-09T15:10:00.000'
                }
            ],
            interfaces: [
                "udm.SourceArtifact",
                "udm.Plop"
            ]
        },
        extraBindingSections: [
            {
                type: 'select',
                label: 'Focus The Field',
                name: 'focusOnField',
                value: '',
                onChangeCallApi: (api, value) => {
                    RU.invoke('focusFieldByName', [value], api);
                },
                values: [
                    'nameComponentString',
                    'nameComponentStringLarge',
                    'nameComponentStringMonaco',
                    'nameComponentInteger',
                    'nameComponentSet',
                    'nameComponentList',
                    'nameComponentCi',
                    'nameStringEnum',
                    'mapStringStringName',
                    'nameSetOfCi',
                    'nameListOfCi',
                    'datetimeName'
                ]
            }
        ],
        navigation: {
            description: 'xl-dip-dynamic-form',
            title: 'DIP Dynamic Form (Warning)',
            type
        }
    },
    dynamicFormAsContainment: {
        formId: 'dynamicFormAsContainment',
        model: {
            id: "Environments/SomeEnv/SomeApp",
            environment: "",
            name: "DeployedApp"
        },
        pristinity: {},
        validity: {},
        bindings,
        metadata: {
            type: "udm.DeployedApplication",
            description: "Description unavailable",
            properties: [
                {
                    name: "environment",
                    label: "environment (asContainment = true)",
                    description: "The environment this deployed app is deployed onto " +
                        "(should be read-only and set even if the _underlying_ model value is null or empty)",
                    kind: "CI",
                    asContainment: true
                }
            ]
        },
        navigation: {
            description: 'xl-react-dip-dynamic-form',
            title: 'DIP Dynamic Form (As Containment)',
            type
        }
    },
    dynamicFormWithCategoryOptions: {
        formId: 'dynamicFormWithCategoryOptions',
        model: {
            environment: ''
        },
        pristinity: {},
        validity: {},
        bindings,
        metadata: {
            categoryOptions: {
                Address: {collapsable: false},
                Common: {titleHidden: true}
            },
            type: "udm.DeployedApplication",
            description: "Description unavailable",
            properties: [
                {
                    description: "The field in the common category, which visually is not categories",
                    kind: "STRING",
                    label: "Name",
                    name: "name"
                },
                {
                    category: 'Address',
                    description: "Address category, which intentionally should be not collapsed",
                    kind: "STRING",
                    label: "City",
                    name: "city"
                }
            ]
        },
        navigation: {
            description: 'xl-react-dip-dynamic-form',
            title: 'DIP Dynamic Form (With Category Options)',
            type
        }
    },
    dynamicFormWithMultipleCategories: {
        formId: 'dynamicFormWithMultipleCategories',
        model: {
            name: "John",
            surname: "Doe"
        },
        pristinity: {},
        validity: {},
        bindings,
        metadata: {
            type: "jea.TopicConnectionFactorySpec",
            description: "Description unavailable",
            properties: [
                {
                    name: "nameComponentString",
                    label: "Label component String",
                    kind: "STRING",
                    description: "Description component String",
                    required: false,
                    password: false,
                    transient: false,
                    size: "DEFAULT"
                },
                {
                    name: "nameComponentStringLarge",
                    label: "Label component StringLarge",
                    kind: "STRING",
                    description: "Description component StringLarge",
                    required: false,
                    password: false,
                    transient: false,
                    size: "LARGE",
                    default: "large string"
                },
                {
                    name: "nameComponentStringMonaco",
                    label: "Label component StringMonaco",
                    kind: "STRING",
                    description: "Description component StringMonaco",
                    required: false,
                    password: false,
                    transient: false,
                    view: "monaco",
                    default: "monaco string"
                },
                {
                    "category": "Personal Data",
                    name: "name",
                    label: "name",
                    kind: "STRING",
                    description: "Your name",
                    required: true,
                    password: false,
                    transient: false,
                    size: "DEFAULT"
                },
                {
                    "category": "Personal Data",
                    name: "surname",
                    label: "Surname",
                    kind: "STRING",
                    description: "Your surname",
                    required: true,
                    password: false,
                    transient: false
                },
                {
                    "category": "Personal Data",
                    name: "age",
                    label: "Age",
                    kind: "INTEGER",
                    description: "Your age",
                    required: true,
                    password: false,
                    transient: false
                }
            ]
        },
        navigation: {
            description: 'xl-react-dip-dynamic-form',
            title: 'DIP Dynamic Form (Multiple Categories)',
            type
        }
    },
    dynamicWithLookup: {
        formId: 'dynamicWithLookup',
        model: {
            encryptedMap: {
                key: 'secret'
            },
            'external-properties': {
                vault: {
                    key: 'SECRET',
                    kind: 'lookup',
                    provider: 'Configurations/Vault/LookupValueProvider'
                }
            },
            largePassword: '',
            password: ''
        },
        pristinity: {},
        validity: {},
        bindings,
        metadata: {
            type: "jea.TopicConnectionFactorySpec",
            description: "Description unavailable",
            properties: [
                {
                    description: "Password field",
                    inputHint: {rules: []},
                    label: "Password field",
                    kind: "STRING",
                    name: "password",
                    required: true,
                    password: true,
                    transient: false,
                    size: "DEFAULT"
                },
                {
                    description: "Large password field",
                    inputHint: {rules: []},
                    label: "Large password field",
                    kind: "STRING",
                    name: "largePassword",
                    required: false,
                    password: true,
                    transient: false,
                    size: "LARGE"
                },
                {
                    description: "Password field",
                    inputHint: {rules: []},
                    label: "Password field with lookup value",
                    kind: "STRING",
                    name: "vault",
                    required: false,
                    password: true,
                    transient: false,
                    size: "DEFAULT"
                },
                {
                    description: "The storage of encrypted values",
                    label: "Encrypted",
                    kind: "MAP_STRING_STRING",
                    name: "encryptedMap",
                    required: false,
                    password: true,
                    transient: false,
                    size: "DEFAULT"
                }
            ],
            interfaces: [
                "udm.SourceArtifact",
                "udm.Plop"
            ]
        },
        extraBindingSections: [
            {
                type: 'select',
                label: 'Focus The Field',
                name: 'focusOnField',
                value: '',
                onChangeCallApi: (api, value) => {
                    RU.invoke('focusFieldByName', [value], api);
                },
                values: [
                    'field0',
                    'field1',
                    'field2'
                ]
            }
        ],
        navigation: {
            description: 'xl-react-dip-form-with-lookup',
            title: 'DIP Form with Lookup',
            type
        },
        validators: [vaultValidator]
    },
    dynamicReadonlyForm: {
        ...dynamicFormDefault,
        formId: 'dynamicReadonlyForm',
        bindings,
        metadata: {
            ...dynamicFormDefault.metadata,
            properties: R.map((property) => ({...property, readonly: true}), dynamicFormDefault.metadata.properties)
        },
        navigation: {
            description: 'xl-react-dip-form-readonly',
            title: 'DIP Dynamic Form (Readonly)',
            type
        }
    }
};
