import R from 'ramda';
import React, {Component} from 'react';
import TestComponent from './test-component';
import ElementNavTitle from './element-nav-title';
import kitchenTestComponentProperties from './kitchen-test-component-props';

export default class WidgetKitchenTestComponent extends Component {
    static propTypes = kitchenTestComponentProperties;

    static defaultProps = {
        disabledToggles: [],
        options: {copyModel: true},
        showBindings: true
    };

    isRequired = (model) => R.pathOr(false, ['metadata', 'required'], model);

    render() {
        const {
            className,
            currentState,
            children,
            label,
            modelName,
            options,
            showBindings,
            updateState
        } = this.props;
        const model = currentState[modelName];
        const {
            api,
            bindings,
            componentProps,
            navigation: {
                title: navigationTitle,
                description: navigationDescription,
                type: navigationType
            }
        } = model;

        const message = R.pathOr({show: false}, ['message'], componentProps);
        const showMessages = R.pathOr(message.show, ['metadata', 'showMessages'], componentProps);
        message.show = showMessages;

        const onUpdateProps = (changedParams, updatedBinding) => {
            const updatedBindings = R.map(R.when(
                R.propEq('name', updatedBinding.name),
                R.assoc('value', updatedBinding.value)
            ))(bindings);

            updateState(modelName, {
                bindings: updatedBindings,
                componentProps: {
                    ...changedParams,
                    message: {}
                }
            });
        };

        const onModelChange = (val) => {
            updateState(modelName, {componentProps: {model: val}});
        };

        const onMessageChange = R.F;

        const onPristinityChange = (val) => {
            updateState(modelName, {componentProps: {pristine: val}});
        };

        const onValidityChange = (val) => {
            updateState(modelName, {componentProps: {valid: val}});
        };

        const onRegister = (val) => {
            updateState(modelName, {api: val});
        };

        const {copyModel} = options;

        const getOnModelChange = () => copyModel ? {onModelChange} : {};

        const getModel = () => copyModel ? {model: componentProps.model} : {};

        const getComponentProps = () => copyModel ? componentProps : R.omit(['model'], componentProps);

        const handleCloseModal = (propName) => () => onUpdateProps({[propName]: false}, {name: propName, value: false});
        const getHandleClose = () =>
            R.isNil(componentProps.showModal) ? {} : {handleClose: handleCloseModal('showModal')};
        const getHandleCloseConfirmationModal = () => R.isNil(componentProps.show) ? {} : {
            onButtonClick: handleCloseModal('show'),
            onCheckboxClick: handleCloseModal('show')
        };

        const createNewParams = () => ({
            ...getComponentProps(),
            message,
            onMessageChange,
            ...getOnModelChange(),
            ...getModel(),
            ...getHandleClose(),
            ...getHandleCloseConfirmationModal(),
            onPristinityChange,
            onValidityChange,
            register: onRegister
        });

        const childrenWithProps = React.Children.map(children,
            (child) => React.cloneElement(child, createNewParams())
        );

        return <div>
            <ElementNavTitle
                description={navigationDescription}
                modelName={modelName}
                title={navigationTitle}
                type={navigationType}
            />
            <h3>{label}</h3>
            <TestComponent
                api={api}
                bindings={bindings}
                className={className}
                componentProps={componentProps}
                modelName={modelName}
                showBindings={showBindings}
                updateProps={onUpdateProps}>
                {childrenWithProps}
            </TestComponent>
        </div>;
    }
}
