import R from 'ramda';
import widgetActions from '../actions/widget-actions';
import {widgetInitialState} from '../widget-page/widget-initial-state';
import {actionReducer, composeReducer, stateIdentity} from './reducer-utils';

const updateModelState = actionReducer((state, action) => {
    const merged = R.mergeDeepRight(state[action.modelName], action.value);
    const assocProp = (propName) => (obj) => {
        const messagePath = ['componentProps', propName];
        const prop = R.path(messagePath, action.value);
        return R.isNil(prop) ? obj : R.assocPath(messagePath, prop, obj);
    };
    const newState = R.pipe(
        assocProp('message'),
        assocProp('model')
    )(merged);
    return {...state, [action.modelName]: newState};
});

export const widgetReducer = composeReducer(
    updateModelState(widgetActions.UPDATE_MODEL_STATE),
    stateIdentity
)(widgetInitialState);
