import R from 'ramda';

const toArray = (param) => Array.isArray(param) ? param : [param];

const invoke = R.curry((path, params, object) =>
    object && R.pipe(R.pathOr(R.F, toArray(path)), R.apply(R.__, params))(object));

const mapIndexed = R.addIndex(R.map);

const indexedArray = mapIndexed((val, key) => [val, key]);

const incrementProp = R.curry((prop, object) => R.over(R.lensProp(prop), R.pipe(R.defaultTo(0), R.inc), object));

const countHits = R.curry((spec, data) => {
    const conditions = R.toPairs(spec);
    const increment = (count, item) => R.reduce(
        (accumulator, [key, condition]) => condition(item) ? incrementProp(key, accumulator) : accumulator,
        count,
        conditions
    );
    return R.reduce(increment, R.map(R.always(0), spec), data);
});

const findDuplicateElement = (items = []) => {
    const moreThanOne = (count) => count > 1;
    const multiples = R.filter(moreThanOne, R.countBy(R.identity, R.pluck('value')(items)));
    return R.head(R.keys(multiples));
};

const areAllTrue = (obj) => R.all(R.identity, R.values(obj));

const isTrue = (path, object) => R.pathEq(path, true)(object);

const isFalse = (path, object) => R.pipe(
    R.pathOr(false, path),
    R.equals(false)
)(object);

const isDefined = R.complement(R.isNil);

const areDefinedAndDifferent = (val1, val2) => R.and(
    R.and(isDefined(val1), isDefined(val2)),
    !R.equals(val1, val2)
);

const removeEmptyProperty = (path) => R.ifElse(
    R.pipe(
        R.pathOr([], path),
        R.isEmpty()
    ),
    R.dissocPath(path),
    R.identity
);

export default {
    areAllTrue,
    areDefinedAndDifferent,
    countHits,
    findDuplicateElement,
    incrementProp,
    indexedArray,
    invoke,
    isDefined,
    isFalse,
    isTrue,
    mapIndexed,
    removeEmptyProperty,
    toArray
};
