import React, {PureComponent} from 'react';
import PropTypes from 'prop-types';
import {MenuItem} from 'react-bootstrap';
import {nextId} from '../../utils/id-generator';
import {clearTimeouts, delayedHandler, labelToClassName} from '../xl-react-button-utils';

export class XlReactButtonDropdownItem extends PureComponent {
    static propTypes = {
        action: PropTypes.func.isRequired,
        disabled: PropTypes.bool,
        header: PropTypes.bool,
        inactiveTime: PropTypes.number,
        labelRenderer: PropTypes.func,
        onSelect: PropTypes.func,
        separator: PropTypes.bool
    };

    static defaultProps = {disabled: false, inactiveTime: 500};

    state = {clickable: true};

    UNSAFE_componentWillMount() {
        this.eventKey = nextId();
        this.timeouts = [];
    }

    componentWillUnmount() {
        clearTimeouts(this.timeouts);
    }

    handleClick = (event) => {
        if (!this.props.disabled) {
            delayedHandler(this, 'action', event);
            this.props.onSelect(false);
        }
    };

    render() {
        const {disabled, header, labelRenderer, separator} = this.props;
        const renderChildren = (sep) => sep ? null : labelRenderer();
        const displayedElement = renderChildren(separator);

        return <MenuItem
            className={labelToClassName(displayedElement)}
            disabled={disabled}
            divider={separator}
            eventKey={this.eventKey}
            header={header}
            key={this.eventKey}
            onClick={this.handleClick}>{displayedElement}
        </MenuItem>;
    }
}
