import React, {Component} from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';
import {DropdownButton, SplitButton} from 'react-bootstrap';
import classNames from 'classnames';
import uuid from 'uuid-v4';
import RU from '../../utils/ramda-utils';
import {clearTimeouts, delayedHandler, labelToClassName} from '../xl-react-button-utils';
import {XlReactButtonDropdownItem} from './xl-react-button-dropdown-item';

import './xl-react-button-with-dropdown.less';

export class XlReactButtonWithDropdown extends Component {
    static propTypes = {
        actions: PropTypes.shape({
            action: PropTypes.func,
            labelRenderer: PropTypes.func.isRequired,
            subActions: PropTypes.arrayOf(PropTypes.shape({
                action: PropTypes.func.isRequired,
                disabled: PropTypes.bool,
                labelRenderer: PropTypes.func.isRequired
            })).isRequired
        }).isRequired,
        className: PropTypes.string,
        disabled: PropTypes.bool,
        id: PropTypes.string,
        inactiveTime: PropTypes.number,
        type: PropTypes.oneOf(['primary', 'secondary']).isRequired
    };

    static defaultProps = {disabled: false, inactiveTime: 500};

    constructor(props) {
        super(props);
        this.state = {
            clickable: true,
            id: this.props.id || uuid(),
            open: false
        };
    }

    UNSAFE_componentWillMount() {
        this.timeouts = [];
    }

    componentWillUnmount() {
        clearTimeouts(this.timeouts);
    }

    handleOnClick = (event) => {
        delayedHandler(this, ['actions', 'action'], event);
    };

    handleToggleOpen = (evt) => {
        this.setState({open: evt});
    };

    render() {
        const {actions: {labelRenderer, subActions}, className, type} = this.props;
        const {id} = this.state;

        const isSplitButton = !R.isNil(R.path(['actions', 'action'], this.props));
        const buttonTitle = labelRenderer();

        const classes = classNames(
            labelToClassName(buttonTitle),
            'xl-react-button',
            `xl-${type}`,
            {'drop-down-only': !isSplitButton}
        );

        const buildDropDownItem = (config, index) => {
            const {
                disabled,
                header,
                labelRenderer: itemLabelRenderer,
                action,
                separator
            } = config;

            return <XlReactButtonDropdownItem
                action={action}
                disabled={disabled}
                header={header}
                key={index}
                labelRenderer={itemLabelRenderer}
                onSelect={this.handleToggleOpen}
                separator={separator}
            />;
        };

        const menu = RU.mapIndexed(buildDropDownItem)(subActions);

        const renderSplitButton = () => <SplitButton
            className={classes}
            disabled={this.props.disabled}
            id={id}
            key={id}
            onClick={this.handleOnClick}
            title={buttonTitle}>
            {menu}
        </SplitButton>;

        const renderDropDownButton = () => <DropdownButton
            className={classes}
            disabled={this.props.disabled}
            id={id}
            key={id}
            onToggle={this.handleToggleOpen}
            open={this.state.open}
            title={buttonTitle}>
            {menu}
        </DropdownButton>;

        const wrapperClasses = classNames(
            'xl-react-button-with-dropdown',
            className,
            type
        );

        return <span className={wrapperClasses}>
            {isSplitButton ? renderSplitButton() : renderDropDownButton()}
        </span>;
    }
}
