import PropTypes from 'prop-types';
import React, {Component} from 'react';
import moment from 'moment';
import RU from '../../utils/ramda-utils';
import {XlReactDipContainerProps} from '../xl-react-dip-props';
import {XlReactDipDatetimeComponent} from './xl-react-dip-datetime-component';

export class XlReactDipDatetime extends Component {
    static propTypes = {
        model: PropTypes.object,
        ...XlReactDipContainerProps
    };
    static defaultProps = {
        metadata: {},
        validators: []
    };

    constructor(props) {
        super(props);
        this.apiData = {};
    }

    handleDateChange = (newDate) => {
        const newDateTime = moment(this.props.model);
        if (newDate && newDateTime.isValid()) {
            newDateTime.date(newDate.getDate());
            newDateTime.month(newDate.getMonth());
            newDateTime.year(newDate.getFullYear());
            this.setNewDateTime(newDateTime.toDate());
        } else {
            this.setNewDateTime(newDate);
        }
    };

    handleTimeChange = (newTime) => {
        const newDateTime = moment(this.props.model);
        if (moment(newTime).isValid()) {
            newDateTime.milliseconds(newTime.getMilliseconds());
            newDateTime.seconds(newTime.getSeconds());
            newDateTime.minutes(newTime.getMinutes());
            newDateTime.hours(newTime.getHours());
            this.setNewDateTime(newDateTime.toDate());
        }
    };

    api = {focus: () => this.apiData.date.focus()};

    register = (component, componentApi) => {
        this.apiData = {
            ...this.apiData,
            [component]: componentApi
        };
        RU.invoke(['register'], [this.api], this.props);
    };

    setNewDateTime = (newDateTime) => {
        RU.invoke('onModelChange', [newDateTime], this.props);
    };

    render() {
        const {
            displayMode, disabled, model, message,
            metadata: {description, label, required, time}, valid
        } = this.props;

        return <XlReactDipDatetimeComponent {...({
            description,
            disabled,
            displayMode,
            handleDateChange: this.handleDateChange,
            handleTimeChange: this.handleTimeChange,
            label,
            message,
            model,
            register: this.register,
            required,
            time,
            valid
        })}
        />;
    }
}
