import R from 'ramda';
import {Promise} from 'es6-promise';
import variables from '../../styles/variables';

export const enumViews = {
    DEFAULT: 'dropdown',
    RADIO: 'radio'
};

export const getHandlers = (isInputHintsPresent, enumValues) => ({
    addCandidates: ({value}) => {
        if (isInputHintsPresent) {
            const hasSearchedTerm = (enumVal) =>
                R.includes(R.toLower(value), R.toLower(R.prop('label', enumVal)));
            const visibleItems = R.filter(hasSearchedTerm, enumValues);
            return new Promise((resolve) => {
                resolve(visibleItems.sort());
            });
        }

        const hasSearchedTerm = (enumVal) => R.includes(R.toLower(value), R.toLower(enumVal));
        return new Promise((resolve) => {
            const visibleItems = R.filter(hasSearchedTerm, enumValues);
            resolve(R.map((name) => ({id: name}))(visibleItems.sort()));
        });
    }
});

export const createRadioGroupModel = (enumValues, model) => R.map((item) => ({
    checked: R.equals(item, model),
    color: variables['xl-blue'],
    label: item,
    value: item
}))(enumValues);

const onSuggestionSelection = (isInputHintsPresent, onModelChange, model, initial, enumValues) => {
    const componentModel = isInputHintsPresent ?
        R.pipe(
            R.find(R.propEq('label', model)),
            R.prop('value')
        )(enumValues) :
        model;
    onModelChange(componentModel, initial);
};

export const handleModelChange = (view, isInputHintsPresent, enumValues, onModelChange) =>
    (componentModel, initial) => {
        const model = view === enumViews.RADIO ?
            R.pipe(R.find(R.prop('checked')), R.prop('value'))(componentModel) : componentModel;
        const isSelectedFromSuggestions = isInputHintsPresent ?
            R.findIndex(R.propEq('label', model))(enumValues) > -1 : R.includes(model, enumValues);

        if (isSelectedFromSuggestions) {
            onSuggestionSelection(isInputHintsPresent, onModelChange, model, initial, enumValues);
        } else {
            onModelChange(model, initial);
        }
    };
