import R from 'ramda';
import React, {PureComponent} from 'react';
import PropTypes from 'prop-types';
import {OverlayTrigger, Popover} from 'react-bootstrap';
import {localeMessages} from '../../services/locale-service';
import RU from '../../utils/ramda-utils';
import {XlReactWidgetInputAddon} from '../../xl-react-widget/xl-react-widget-input-addon/xl-react-widget-input-addon';
import {XlReactWidgetInputAddonGroup} from
    '../../xl-react-widget/xl-react-widget-input-addon/xl-react-widget-input-addon-group';
import {XlReactWidgetSwitch} from '../../xl-react-widget/xl-react-widget-switch/xl-react-widget-switch';
import {XlReactWidgetSwitchButton} from
    '../../xl-react-widget/xl-react-widget-switch/xl-react-widget-switch-button/xl-react-widget-switch-button';
import {XlReactDipCi} from '../xl-react-dip-ci/xl-react-dip-ci';
import {XlReactDipContainerProps} from '../xl-react-dip-props';
import {XlReactDipString} from '../xl-react-dip-string/xl-react-dip-string';
import {lookupModelType} from './xl-react-dip-lookup-type';
import {apiFocus} from './xl-react-dip-lookup-service';

import './xl-react-dip-lookup.less';

const omitRegister = (props) => R.omit(['register'], props);

export class XlReactDipLookup extends PureComponent {
    static propTypes = {
        fieldComponent: PropTypes.func.isRequired,
        handlers: PropTypes.object,
        model: lookupModelType,
        onLookupToggle: PropTypes.func.isRequired,
        ...XlReactDipContainerProps
    };

    constructor(props) {
        super(props);
        RU.invoke('register', [this.api], props);
    }

    handleLookupChange = R.curry((field, value) => {
        RU.invoke('onModelChange', [R.assoc(field, value, this.props.model)], this.props);
    });

    handleProviderChange = (value, updatePristinity = true) => {
        if (!updatePristinity) {
            return;
        }
        RU.invoke(['key', 'markAsDirty'], [], this.componentApis);
        this.handleLookupChange('provider')(value);
    };

    api = {focus: () => apiFocus(this)};

    componentApis = {};

    lookupClassName = 'xl-react-dip-lookup';

    lookupPopover = <Popover
        className="tooltip-popover"
        id="xl-react-dip-lookup-popover">
        {localeMessages.dipLookupToggleTooltipMessage}
    </Popover>;

    registerApi = (field) => (api) => {
        this.componentApis[field] = api;
        RU.invoke('register', [this.api], this.props);
    };

    unregisterApi = (field) => () => {
        this.componentApis = R.omit([field], this.componentApis);
        RU.invoke('register', [this.api], this.props);
    };

    isLookupField = () => R.propSatisfies(R.is(Object), 'model', this.props);

    renderSwitch = () => {
        const {onLookupToggle, disabled} = this.props;
        return <XlReactWidgetInputAddonGroup>
            <XlReactWidgetInputAddon>
                <XlReactWidgetSwitch
                    disabled={disabled}
                    independent={true}
                    model={this.isLookupField()}
                    onModelChange={onLookupToggle}
                    renderSwitch={({className}) =>
                        <XlReactWidgetSwitchButton className={className}>
                            <OverlayTrigger
                                overlay={this.lookupPopover}
                                placement="top"
                                trigger={['focus', 'hover', 'click']}>
                                <i className="xl-icon keyboard-icon"/>
                            </OverlayTrigger>
                        </XlReactWidgetSwitchButton>
                    }
                />
            </XlReactWidgetInputAddon>
        </XlReactWidgetInputAddonGroup>;
    };

    renderLookupFields = () => {
        const {
            disabled,
            displayMode,
            handlers,
            message,
            metadata,
            model: {key, provider},
            valid
        } = this.props;

        const required = metadata.required || Boolean(key) || Boolean(provider);

        const providerMetadata = {
            ...metadata,
            asRequired: required,
            description: undefined
        };
        const keyMetadata = {
            ...metadata,
            asRequired: required,
            label: ' ',
            password: false,
            required: false,
            size: 'DEFAULT',
            type: 'text'
        };

        return <React.Fragment>
            <XlReactDipCi
                className={this.lookupClassName}
                controlClassName="xl-react-dip-lookup-control"
                disabled={disabled}
                displayMode={displayMode}
                handlers={handlers}
                metadata={providerMetadata}
                model={provider}
                onModelChange={this.handleProviderChange}
                register={this.registerApi('provider')}
                showSearchIcon={false}
                toggleButton={this.renderSwitch()}
                unregister={this.unregisterApi('provider')}
            />
            <XlReactDipString
                disabled={disabled}
                displayMode={displayMode}
                message={message}
                metadata={keyMetadata}
                model={key}
                onModelChange={this.handleLookupChange('key')}
                register={this.registerApi('key')}
                unregister={this.unregisterApi('key')}
                valid={valid}
            />
        </React.Fragment>;
    };

    render() {
        const {fieldComponent: FieldComponent, ...rest} = this.props;

        if (this.isLookupField()) {
            this.unregisterApi('baseField');
            return this.renderLookupFields();
        }

        this.unregisterApi('key');
        this.unregisterApi('provider');
        return <FieldComponent
            {...omitRegister(rest)}
            className={this.lookupClassName}
            register={this.registerApi('baseField')}
            toggleButton={this.renderSwitch()}
            unregister={this.unregisterApi('baseField')}
        />;
    }
}
