import R from 'ramda';
import classNames from 'classnames';
import {defaultTableRowRenderer} from 'react-virtualized';
import {sortStrings} from '../../utils/sorting-utils';
import RU from '../../utils/ramda-utils';

export const HEADER_HEIGHT = 26;
export const ROW_HEIGHT = 29;
export const MAX_ROWS = 5;
export const COLUMN_MIN_WIDTH = 100;
export const COLUMN_COUNT = 2;
export const COLUMNS = {
    key: 'key',
    value: 'value'
};
export const NEW_VALUE_CELL_KEY = 'NEW_VALUE_CELL_KEY';

export const getRowClassNames = ({index}) => classNames(
    'xl-virtualized-table-row',
    {'xl-virtualized-table-header': index === -1},
    {'-odd': index !== -1 && (index + 1) % 2 !== 0}
);

export const calculateMapTableHeight = (maxRows, rowsHeight, headerHeight) => {
    const maxHeight = maxRows * rowsHeight;
    return maxHeight + headerHeight;
};

export const rowGetter = (data) => ({index}) => data[index];

export const fillDataWithBlankRows = (data) => {
    const dataLength = R.length(data);
    if (dataLength < MAX_ROWS) {
        return R.concat(data, R.repeat({}, MAX_ROWS - dataLength));
    }
    return R.identity(data);
};

export const rowRenderer = R.cond([
    [
        R.compose(R.isEmpty, R.path(['rowData'])),
        R.pipe(
            R.evolve({className: (className) => classNames(className, 'row-placeholder')}),
            defaultTableRowRenderer
        )
    ],
    [R.T, defaultTableRowRenderer]
]);

export const preventEvent = (event) => {
    event.stopPropagation();
};

export const calculateColumnResizerBounds = (tableWidth) => {
    const bound = tableWidth / 2 - COLUMN_MIN_WIDTH;
    return {
        left: -1 * bound,
        right: bound
    };
};

export const shouldFocusInput = (row, column, focusState) => focusState ? R.whereEq({column, row}, focusState) : false;

export const getUpdatedCellState = (model, currentCellState, {sortBy, sortDirection}) => {
    const values = sortBy === 'key' ? R.keys(model) : R.values(model);
    const sortedValues = sortStrings(values, sortDirection);
    return currentCellState ? {
        ...currentCellState,
        value: sortedValues[currentCellState.row]
    } : currentCellState;
};

export const appendNewKeyValueRow = (showNewKeyValueRow, data) => {
    if (showNewKeyValueRow) {
        return R.prepend({key: undefined, value: undefined}, data);
    }
    return R.identity(data);
};

export const updateState = (context, nextProps) => {
    const update = () => {
        const setState = (internalValue) => {
            context.setState({internalValue});
        };

        if (RU.areDefinedAndDifferent(nextProps.changedValue, nextProps.value)) {
            setState(nextProps.changedValue);
        } else if (!R.isNil(nextProps.value)) {
            setState(nextProps.value);
        }
    };

    if (context.props != nextProps) { // eslint-disable-line eqeqeq
        update();
    }
};
