import React, {Component} from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';
import RU from '../../utils/ramda-utils';
import {updateState} from '../xl-react-dip-map-string-string/map-table-utils';
import {blurFieldOnSubmit} from '../input/input-service';

export class MapKeyInput extends Component {
    static propTypes = {
        assignRef: PropTypes.func,
        changedValue: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
        className: PropTypes.string,
        disabled: PropTypes.bool.isRequired,
        handleRemoveKey: PropTypes.func.isRequired,
        onBlur: PropTypes.func,
        onChange: PropTypes.func,
        onKeyDown: PropTypes.func,
        placeholder: PropTypes.string,
        readOnly: PropTypes.bool,
        shouldFocus: PropTypes.bool,
        showRemoveButton: PropTypes.bool.isRequired,
        updateCellState: PropTypes.func,
        value: PropTypes.string.isRequired
    };

    constructor(props) {
        super(props);
        const {changedValue, value} = props;
        if (!R.isNil(changedValue) && value !== changedValue) {
            this.state = {internalValue: props.changedValue};
        } else {
            this.state = {internalValue: props.value};
        }
    }

    UNSAFE_componentWillReceiveProps(nextProps) {
        updateState(this, nextProps);
    }

    componentWillUnmount() {
        const {internalValue} = this.state;
        const {value} = this.props;
        if (!R.equals(internalValue, value)) {
            this.props.onBlur(internalValue, value);
        }
    }

    handleBlur = (event) => {
        this.props.onBlur(this.state.internalValue, this.props.value, event.target);
    };

    handleFocus = () => {
        this.props.updateCellState(this.state.internalValue);
    };

    handleValueChange = (event) => {
        RU.invoke('onChange', [event], this.props);
        this.setState({internalValue: event.target.value}, () => {
            this.props.updateCellState(this.state.internalValue);
        });
    };

    handleKeyDown = (event) => {
        blurFieldOnSubmit(this.inputField, event);
    };

    handleRemoveKey = () => {
        const {handleRemoveKey, value} = this.props;
        handleRemoveKey(value);
    };

    assignRef = (input) => {
        this.inputField = input;
        RU.invoke('assignRef', [input], this.props);
    };

    render() {
        const {readOnly, placeholder, className, disabled, showRemoveButton} = this.props;
        const {internalValue} = this.state;

        const closeBtn = <i
            className="xl-icon close-icon"
            onClick={this.handleRemoveKey}
        />;

        return <div className="key-div">
            <input
                className={className}
                disabled={disabled}
                onBlur={this.handleBlur}
                onChange={this.handleValueChange}
                onFocus={this.handleFocus}
                onKeyDown={this.handleKeyDown}
                placeholder={placeholder}
                readOnly={readOnly}
                ref={this.assignRef}
                title={internalValue}
                type="text"
                value={internalValue}
            />
            {disabled || showRemoveButton ? null : closeBtn}
        </div>;
    }
}
