import React, {Component} from 'react';
import PropTypes from 'prop-types';
import {Table, AutoSizer, SortDirection} from 'react-virtualized';
import {xlMessageTypeProp} from '../../xl-props/xl-message-type-prop';
import {
    calculateMapTableHeight,
    getRowClassNames, rowGetter, rowRenderer
} from '../xl-react-dip-map-string-string/map-table-utils';
import {isError} from '../../utils/ci-utils';
import {MAX_ROWS, HEADER_HEIGHT, ROW_HEIGHT, fillDataWithBlankRows} from './set-of-strings-utils';
import {createHeaderRowRenderer} from './header-row-renderer';
import {tableKeyColumn} from './columns/table-key-column';
import 'react-virtualized/styles.css';

export default class SetOfStringsTable extends Component {
    static propTypes = {
        assignRef: PropTypes.func.isRequired,
        assignTableRef: PropTypes.func.isRequired,
        cellState: PropTypes.shape({
            column: PropTypes.oneOf(['key']),
            row: PropTypes.number.isRequired,
            value: PropTypes.oneOfType([PropTypes.string, PropTypes.number]).isRequired
        }),
        data: PropTypes.array.isRequired,
        disabled: PropTypes.bool.isRequired,
        handleCloseButton: PropTypes.func.isRequired,
        handleKeyBlur: PropTypes.func.isRequired,
        handleNewKeyBlur: PropTypes.func.isRequired,
        handleNewKeyChange: PropTypes.func.isRequired,
        handleNewKeyFocus: PropTypes.func.isRequired,
        handleRemoveKey: PropTypes.func.isRequired,
        hasKeyError: PropTypes.bool.isRequired,
        message: xlMessageTypeProp,
        newKey: PropTypes.string.isRequired,
        onSortChange: PropTypes.func.isRequired,
        showNewKeyValueRow: PropTypes.bool.isRequired,
        sortBy: PropTypes.oneOf(['key']),
        sortDirection: PropTypes.oneOf([SortDirection.ASC, SortDirection.DESC]),
        updateCellState: PropTypes.func.isRequired,
        valid: PropTypes.bool.isRequired
    };

    render() {
        const {
            data,
            disabled,
            onSortChange,
            sortBy,
            sortDirection,
            assignTableRef,
            handleCloseButton,
            handleRemoveKey,
            message,
            showNewKeyValueRow,
            handleKeyBlur,
            hasKeyError,
            assignRef,
            handleNewKeyBlur,
            handleNewKeyChange,
            handleNewKeyFocus,
            newKey,
            valid,
            cellState,
            updateCellState
        } = this.props;

        const filledRowsData = fillDataWithBlankRows(data);
        const tableHeight = calculateMapTableHeight(MAX_ROWS, ROW_HEIGHT, HEADER_HEIGHT);

        return (
            <AutoSizer disableHeight={true} disableWidth={false}>
                {({width}) =>
                    <Table
                        className="-striped -highlight table-scrollable-container xl-virtualized-table"
                        data={filledRowsData}
                        gridClassName="xl-virtualized-table-body"
                        headerClassName="xl-virtualized-table-header-cell"
                        headerHeight={HEADER_HEIGHT}
                        headerRowRenderer={createHeaderRowRenderer(tableHeight, width)}
                        height={tableHeight}
                        ref={assignTableRef}
                        rowClassName={getRowClassNames}
                        rowCount={filledRowsData.length}
                        rowGetter={rowGetter(filledRowsData)}
                        rowHeight={ROW_HEIGHT}
                        rowRenderer={rowRenderer}
                        sort={onSortChange}
                        sortBy={sortBy}
                        sortDirection={sortDirection}
                        width={width}>
                        {tableKeyColumn({
                            assignRef,
                            cellState,
                            columnWidth: width,
                            disabled,
                            handleCloseButton,
                            handleKeyBlur,
                            handleNewKeyBlur,
                            handleNewKeyChange,
                            handleNewKeyFocus,
                            handleRemoveKey,
                            hasKeyError,
                            invalid: isError(valid, message) || hasKeyError,
                            newKey,
                            showNewKeyValueRow,
                            updateCellState
                        })}
                    </Table>
                }
            </AutoSizer>
        );
    }
}
