import R from 'ramda';
import {SortDirection} from 'react-virtualized';

export const getModelKeys = (modelValue, pristine) => R.pipe(
    pristine ? R.sortBy(R.identity) : R.identity
)(modelValue);

export const filterModel = (modelValue, searchToken, isEncrypted) => {
    const matchesTerm = R.pipe(
        R.toString,
        R.toLower,
        R.includes(R.toLower(searchToken))
    );

    const searchByValue = (value) => isEncrypted ? false : matchesTerm(value);

    return R.filter(searchByValue, modelValue);
};

export const getModelWithUpdatedKey = (model, originalKey, newKey) =>
    R.indexOf(originalKey, model) !== -1 ?
        R.update(R.indexOf(originalKey, model), newKey, model) : model;

export const getModelKeysWithUpdatedKey = (modelKeys, originalKey, newKey) => R.reduce(
    (acc, key) => key === originalKey ? [...acc, R.trim(newKey)] : [...acc, key], [], modelKeys);

export const tableSortComparator = (current, next, desc) => {
    const comparator = R.comparator(desc ? R.gt : R.lt);
    return R.isNil(current) ? Number(desc) : comparator(current, next);
};

export const comparatorBuilder = (getSortValue, desc) =>
    (object1, object2) => tableSortComparator(getSortValue(object1), getSortValue(object2), desc);

export const sortData = (data, sortDirection) => {
    if (R.isNil(sortDirection)) {
        return R.identity(data);
    }

    const comparator = comparatorBuilder(
        R.pipe(
            R.cond([
                [R.isNil, R.identity],
                [R.T, String]
            ])
        ),
        sortDirection === SortDirection.DESC
    );
    return R.sort(comparator, data);
};
