import classNames from 'classnames';
import PropTypes from 'prop-types';
import R from 'ramda';
import React, {PureComponent} from 'react';
import {AutoSizer, List} from 'react-virtualized';

export class XlReactWidgetAutocompleteMenu extends PureComponent {
    static propTypes = {
        clazzName: PropTypes.string,
        highlightedSuggestionIndex: PropTypes.number,
        menuHeight: PropTypes.number,
        onSuggestionClick: PropTypes.func,
        onSuggestionMouseDown: PropTypes.func,
        optionHeight: PropTypes.number,
        suggestionRenderer: PropTypes.func.isRequired,
        suggestions: PropTypes.array,
        suggestionsContainerRenderer: PropTypes.func,
        value: PropTypes.oneOfType([
            PropTypes.string,
            PropTypes.object
        ])
    };

    static defaultProps = {suggestionsContainerRenderer: R.identity};

    render() {
        const {
            menuHeight,
            optionHeight,
            suggestions = [],
            suggestionsContainerRenderer
        } = this.props;

        const rowRenderer = ({key, index, style}) => {
            const {
                onSuggestionClick,
                onSuggestionMouseDown,
                clazzName,
                suggestionRenderer,
                value
            } = this.props;

            const suggestion = suggestions[index];
            const eventListeners = {
                onClick: R.partialRight(onSuggestionClick, [suggestion]),
                onMouseDown: R.partialRight(onSuggestionMouseDown, [suggestion])
            };
            const suggestionClassNames = classNames(
                {
                    'xl-react-autocomplete-suggestion': true,
                    'xl-react-autocomplete-suggestion--highlighted': R.equals(suggestion, value)
                },
                clazzName
            );
            return (
                <div
                    className={suggestionClassNames}
                    data-suggestion-index={index}
                    key={key}
                    style={style}
                    {...eventListeners}>
                    {suggestionRenderer(suggestion, value)}
                </div>
            );
        };

        const listRender = suggestions.length > 0 ?
            <AutoSizer disableHeight={true} disableWidth={false}>
                {
                    ({width}) =>
                        <List
                            className="xl-react-autocomplete-suggestion-list"
                            height={menuHeight}
                            rowCount={suggestions.length}
                            rowHeight={optionHeight}
                            rowRenderer={rowRenderer}
                            width={width}
                        />
                }
            </AutoSizer> : null;

        return suggestionsContainerRenderer(listRender);
    }
}
