import React, {PureComponent} from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';
import {Markup} from 'interweave';
import {connect} from 'react-redux';
import classNames from 'classnames';
import {XlReactWidgetConfirmationDialog}
    from '../xl-react-widget-confirmation-dialog/xl-react-widget-confirmation-dialog';
import confirmationDialogActions
    from './redux/confirmation-dialog-actions';

const mapStateToProps = (state) => ({dialogState: R.prop('confirmationDialog', state)});

const mapDispatchToProps = (dispatch) => ({handleClose: (reason) => dispatch(confirmationDialogActions.hide(reason))});

@connect(mapStateToProps, mapDispatchToProps)
export class XlReactWidgetModalDialogContainer extends PureComponent {
    static propTypes = {
        dialogState: PropTypes.object,
        handleClose: PropTypes.func
    };

    render() {
        const {
            dialogState,
            handleClose
        } = this.props;

        const buttons = R.path(['options', 'buttons'], dialogState);
        const message = R.path(['options', 'message'], dialogState);
        const dialogClassName = classNames(
            'xl-react-widget-confirmation-dialog',
            R.path(['options', 'dialogClassName'], dialogState)
        );
        const title = R.path(['options', 'title'], dialogState);

        const isShown = R.pathOr(false, ['show'], dialogState);

        return <XlReactWidgetConfirmationDialog
            buttons={buttons}
            dialogClassName={dialogClassName}
            onButtonClick={handleClose}
            show={isShown}
            title={title}>
            <Markup content={message}/>
        </XlReactWidgetConfirmationDialog>;
    }
}
