import React, {Component} from 'react';
import $ from 'jquery';
import R from 'ramda';
import PropTypes from 'prop-types';
import className from 'classnames';
import RU from '../../utils/ramda-utils';
import {isComponentDisabled} from '../../utils/components-utils';
import {computeInitialState, computeNextState} from '../../services/component-service';
import {xlMessageTypeProp} from '../../xl-props/xl-message-type-prop';

import './xl-react-widget-radio-group.less';

export class XlReactWidgetRadioGroup extends Component {
    static propTypes = {
        disabled: PropTypes.bool,
        message: xlMessageTypeProp,
        model: PropTypes.arrayOf(
            PropTypes.shape({
                checked: PropTypes.bool,
                color: PropTypes.string.isRequired,
                label: PropTypes.string.isRequired,
                value: PropTypes.any.isRequired
            }).isRequired
        ).isRequired,
        name: PropTypes.string.isRequired,
        onModelChange: PropTypes.func,
        onPristinityChange: PropTypes.func,
        register: PropTypes.func,
        separatorLabel: PropTypes.string
    };

    static defaultProps = {model: []};

    constructor(props) {
        super(props);
        this.state = {model: []};
    }

    UNSAFE_componentWillMount() {
        computeInitialState(this, this.props, [], R.map((item) => ({
            ...item,
            checked: item.checked || false
        }), this.props.model));
    }

    componentDidMount() {
        $(this.radioGroup).on('click', (event) => {
            if (!isComponentDisabled(this.props)) {
                const selectedOption = R.path(['data', 'value'], event.target.attributes);
                if (selectedOption) {
                    const newModel = R.map((item) => ({
                        ...item,
                        checked: selectedOption === item.value
                    }), this.props.model);
                    computeNextState(this, this.props, [], newModel);
                }
            }
        });
    }

    UNSAFE_componentWillReceiveProps(newProps) {
        this.setState({model: newProps.model});
    }

    componentWillUnmount() {
        $(this.radioGroup).off('click');
    }

    api = {focus: () => R.F};

    assignRef = (radioGroup) => {
        this.radioGroup = radioGroup;
    };

    buildInput = (item, index) => {
        const handleNothing = R.F;
        const disabled = isComponentDisabled(this.props);

        const radioClassName = className('xl-react-widget-radio', {disabled});

        const inputLabelClassName = className('input-label', {checked: item.checked});

        return <div className={radioClassName} key={`input-${index}`}>
            <input
                checked={item.checked || false}
                data={item.label}
                disabled={disabled}
                name={this.props.name}
                onChange={handleNothing}
                type="radio"
                value={item.value}
            />
            <label
                className="radio-group-option"
                data={item.label}
                style={item.checked ? {backgroundColor: item.color} : undefined}
            />
            <label
                className={inputLabelClassName}
                data={item.label}>
                {item.label}
            </label>
        </div>;
    };

    buildSeparatorLabel = (index) =>
        <label className="label-element" key={`label-${index}`}>{this.props.separatorLabel}</label>;

    buildItem = (item, index) => {
        const {separatorLabel} = this.props;
        const input = this.buildInput(item, index);
        const isLastElement = index < this.props.model.length - 1;
        const shouldAddSeparator = separatorLabel && isLastElement;
        return shouldAddSeparator ? [input, this.buildSeparatorLabel(index)] : input;
    };

    render() {
        const {model} = this.props;
        return <div className="xl-react-widget-radio-group" ref={this.assignRef}>
            {RU.mapIndexed(this.buildItem, model)}
        </div>;
    }
}
