import React, {PureComponent} from 'react';
import classNames from 'classnames';
import PropTypes from 'prop-types';

import './xl-react-widget-input-component.less';
import {getInputType}
    from '../../xl-react-widget/xl-react-widget-reveal-password-icon/xl-react-widget-reveal-password-utils';
import XlReactWidgetRevealPasswordIcon
    from '../xl-react-widget-reveal-password-icon/xl-react-widget-reveal-password-icon';

export class XlReactWidgetInputComponent extends PureComponent {
    static propTypes = {
        assignRef: PropTypes.func,
        autocomplete: PropTypes.string,
        canRevealPassword: PropTypes.bool,
        className: PropTypes.string,
        disabled: PropTypes.bool,
        inputClasses: PropTypes.string,
        model: PropTypes.string,
        name: PropTypes.string,
        onBlur: PropTypes.func,
        onFocus: PropTypes.func,
        onKeyPress: PropTypes.func,
        onModelChange: PropTypes.func,
        onPaste: PropTypes.func,
        onRevealPasswordToggle: PropTypes.func,
        placeholder: PropTypes.string,
        pristine: PropTypes.bool,
        required: PropTypes.bool,
        revealPassword: PropTypes.bool,
        toggleButton: PropTypes.element,
        type: PropTypes.string
    };

    shouldShowIcon = () => {
        const {canRevealPassword, model, type} = this.props;
        return canRevealPassword && model && type === 'password';
    };

    render() {
        const {
            autocomplete,
            className,
            inputClasses,
            name,
            onBlur,
            onFocus,
            onModelChange,
            onPaste,
            onRevealPasswordToggle,
            onKeyPress,
            placeholder,
            disabled,
            assignRef,
            required,
            model,
            revealPassword,
            toggleButton,
            type
        } = this.props;

        const clazzName = classNames(className, 'xl-react-widget-input-component');

        return <div className={clazzName}>
            {toggleButton}
            <input
                autoComplete={type === 'password' && !autocomplete ? 'new-password' : autocomplete}
                className={inputClasses}
                name={name}
                onBlur={onBlur}
                onChange={onModelChange}
                onFocus={onFocus}
                onKeyPress={onKeyPress}
                onPaste={onPaste}
                placeholder={placeholder}
                readOnly={disabled}
                ref={assignRef}
                required={required}
                type={getInputType(type, revealPassword)}
                value={model}
            />
            <XlReactWidgetRevealPasswordIcon
                handleRevealPassword={onRevealPasswordToggle}
                isPasswordRevealed={revealPassword}
                showIconCondition={this.shouldShowIcon}
            />
        </div>;
    }
}
