import RS from 'ramdasauce';
import React, {Component} from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';
import classNames from 'classnames';
import {localeMessages} from '../../services/locale-service';
import {widgetStringValues} from './xl-react-widget-string-enum';

import './xl-react-widget-multi-line-password-component.less';

export class XlReactWidgetMultiLinePasswordComponent extends Component {
    static propTypes = {
        assignRef: PropTypes.func.isRequired,
        className: PropTypes.string,
        disabled: PropTypes.bool,
        focused: PropTypes.bool,
        inputClasses: PropTypes.string,
        model: PropTypes.string,
        name: PropTypes.string,
        onBlur: PropTypes.func,
        onFocus: PropTypes.func,
        onModelChange: PropTypes.func,
        onStageModel: PropTypes.func.isRequired,
        onUnstageModel: PropTypes.func.isRequired,
        placeholder: PropTypes.string,
        pristine: PropTypes.bool.isRequired,
        required: PropTypes.bool,
        size: PropTypes.string
    };

    handleOnFocus = () => {
        this.props.onStageModel();
        this.props.onFocus();
    };

    handleOnBlur = (event) => {
        this.props.onUnstageModel();
        this.props.onBlur(event);
    };

    getDisplayValue = (model, pristine, showModel) => {
        const valueLabel = pristine ?
            localeMessages.encryptedUnchangedValueLabel : localeMessages.encryptedChangedValueLabel;
        return showModel ? model : valueLabel;
    };

    getDisplaySizeClass = R.cond([
        [
            R.equals(widgetStringValues.LARGE),
            R.always('xl-react-widget-multi-line-password-large-component')
        ],
        [
            R.equals(widgetStringValues.MEDIUM),
            R.always('xl-react-widget-multi-line-password-medium-component')
        ],
        [
            R.T,
            R.always('xl-react-widget-multi-line-password-medium-component')
        ]
    ]);

    render() {
        const {
            assignRef,
            className,
            disabled,
            inputClasses,
            focused,
            model,
            name,
            onModelChange,
            placeholder,
            pristine,
            required,
            size
        } = this.props;
        const showModel = focused || RS.isNilOrEmpty(model);
        const displayedValue = this.getDisplayValue(model, pristine, showModel);
        const sizeClass = this.getDisplaySizeClass(size);

        const divClassName = classNames('xl-react-components', className, sizeClass,
            {focused: !disabled && focused});

        return <div className={divClassName}>
            <textarea
                className={inputClasses}
                disabled={disabled}
                name={name}
                onBlur={this.handleOnBlur}
                onChange={onModelChange}
                onFocus={this.handleOnFocus}
                placeholder={placeholder}
                ref={assignRef}
                required={required}
                value={displayedValue}
            />
        </div>;
    }
}
