import util from 'util';

const allowedNotBarColorValues = ['transparent', 'inherit'];

const properties = ['background-color', 'border-bottom-color', 'border-color', 'border-left-color',
    'border-right-color', 'border-top-color', 'caret-color', 'color', 'column-rule-color', 'outline-color',
    'text-decoration-color'];

const linter = {
    lint: (config, node) => { // eslint-disable-line complexity
        if (!properties.includes(node.prop)) {
            return [];
        }

        const value = node.value.toString().trim();

        let message;

        if (!allowedNotBarColorValues.includes(value) && !value.includes('@')) {
            message = util.format(linter.message.notVariable, value);
        }

        if (message) {
            const position = node.positionBy({word: value});

            return [{
                column: position.column,
                line: position.line,
                message: message
            }];
        }

        return [];
    },
    message: {notVariable: 'Color value (%s) should be specified only as a variable.'},
    name: 'onlyVariableColors',
    nodeTypes: ['decl']
};

export default linter;
