import {
    Action,
    ActionUtil,
    Expectation,
    WaitCondition
} from 'protractor-base-dsl';

const getElementByLabel = (label, parent) => {
    const divPath = './/div[@class=\'xl-react-components-label\']';
    const selector = By.xpath(`${divPath}/label[text()='${label}']/parent::div/parent::div`);
    const labelEl = parent ? parent.element(selector) : element(selector);
    Expectation.displayed(labelEl);
    return labelEl;
};

const getCategoryByLabel = (label, parent) => {
    const divPath = './/div[@class=\'xl-react-widget-category open\']';
    const selector = By.xpath(`${divPath}/div[text()='${label}']/parent::div`);
    const labelEl = parent ? parent.element(selector) : element(selector);
    Expectation.displayed(labelEl);
    return labelEl;
};

const inputByLabel = (label, parent) =>
    getElementByLabel(label, parent)
        .element(By.css('.xl-react-components-input input'));

const textareaByLabel = (label, parent) =>
    getElementByLabel(label, parent)
        .element(By.css('.xl-react-components-input textarea'));

const buttonByLabel = (label, parent) =>
    getElementByLabel(label, parent)
        .element(By.css('.xl-react-components-input button'));

const clearInputByLabel = (label, parent) => {
    const el = getElementByLabel(label, parent);
    Expectation.displayed(el);
    const cleanButton = el.element(By.css('.xl-react-components-input .close-icon'));
    Action.click(cleanButton);
    return Expectation.emptyText(el.element(By.css('input')));
};

const selectTagsFilterSuggestion = (statusName, tagClass) => {
    const filterSelector = `${tagClass} .xl-react-widget-autocomplete .xl-react-autocomplete-suggestion-list`;
    const stateTagsFilterInput = `${tagClass} .xl-react-widget-autocomplete .react-tagsinput-input`;
    const optionSelector = `${filterSelector} .yt-option[title="${statusName}"]`;
    const labelSelector = `${tagClass} .xl-react-components-label`;
    const action = () => Action.click(tagClass);
    const condition = () => WaitCondition.displayed(optionSelector);
    ActionUtil.repeatAction(action, condition);

    Action.typeNewText(element(By.css(stateTagsFilterInput)), statusName);
    Expectation.displayed(optionSelector);
    Expectation.count(optionSelector, 1);
    browser.sleep(500);

    const action2 = () => {
        Action.click(optionSelector);
        Action.click(labelSelector);
    };
    const condition2 = () => WaitCondition.displayed(`${tagClass} .react-tagsinput-tag[title="${statusName}"]`);
    return ActionUtil.repeatAction(action2, condition2);
};

const XlComponentsHelper = {
    buttonByLabel,
    clearInputByLabel,
    getCategoryByLabel,
    getElementByLabel,
    inputByLabel,
    selectTagsFilterSuggestion,
    textareaByLabel
};
export default XlComponentsHelper;
global.XlComponentsHelper = XlComponentsHelper;
