import Navigation from '../../../dsl/common/navigation';
import TestBindings from '../../../dsl/common/test-bindings';
import DipEnum from '../../../dsl/component/dip/dip-enum';

describe('XL DIP Enum Scenario', () => {
    afterAll(() => TestBindings.resetPristinity());
    beforeAll(() => {
        Navigation.openDipElement('dipEnum1');
        DipEnum.toggleDropdownView();
    });

    it('should have right initial state and show all suggestions when pristine', () => {
        TestBindings.expectModelToEqual('eagle');
        TestBindings.expectPristineToEqual('true');
        TestBindings.expectValidToEqual('true');
        DipEnum.expectLabel('Enum Label');
        DipEnum.expectDescription('Select a string');
        DipEnum.openSelection();
        DipEnum.expectSuggestionValue(1, 'ace');
        DipEnum.expectSuggestionValue(2, 'albatross');
        DipEnum.expectSuggestionValue(3, 'birdie');
        DipEnum.expectSuggestionValue(4, 'bogey');
        DipEnum.expectSuggestionValue(5, 'eagle');
    });

    it('should show options and provide search', () => {
        TestBindings.expectModelToEqual('eagle');
        DipEnum.clearSelection();
        DipEnum.openSelection();
        DipEnum.expectSuggestionValue(1, 'ace');
        DipEnum.expectSuggestionValue(2, 'albatross');
        DipEnum.expectSuggestionValue(3, 'birdie');
        DipEnum.expectSuggestionValue(4, 'bogey');
        DipEnum.expectSuggestionValue(5, 'eagle');

        DipEnum.setValue('ss');
        DipEnum.expectSuggestionValue(1, 'albatross');
        DipEnum.selectOption(1);

        TestBindings.expectModelToEqual('albatross');
        TestBindings.expectPristineToEqual('false');
        TestBindings.expectValidToEqual('true');
    });

    it('should be disabled when "disabled" option is chosen', () => {
        DipEnum.expectEnabled();
        TestBindings.toggleDisabled();
        DipEnum.expectClearSelectionNotPresent();
        DipEnum.expectDisabled();
        TestBindings.toggleDisabled();
        DipEnum.expectEnabled();
    });

    it('should display cross if any option is selected', () => {
        DipEnum.expectClearSelectionPresent();
        DipEnum.clearSelection();
        DipEnum.expectClearSelectionNotPresent();
        TestBindings.expectPristineToEqual('false');
        TestBindings.expectValidToEqual('true');
        TestBindings.expectModelToEqual('');

        DipEnum.openSelection();
        DipEnum.selectOption(4);
        TestBindings.expectModelToEqual('bogey');

        DipEnum.clearSelection();
        DipEnum.setValue('e');
        DipEnum.openSelection();
        DipEnum.selectOption(4);
        TestBindings.expectModelToEqual('eagle');
    });
});
