import R from 'ramda';
import Navigation from '../../../dsl/common/navigation';
import TestBindings from '../../../dsl/common/test-bindings';
import DipList from '../../../dsl/component/dip/dip-list';

describe('XL DIP List Without Autocomplete Scenario', () => {
    beforeAll(() => Navigation.openDipElement('dipArrayOfStrings1'));

    it('should have right initial state', () => {
        TestBindings.expectModelToEqual('["Item 1","Item 2","Item 3","Item 4"]');
        TestBindings.expectPristineToEqual('true');
        TestBindings.expectValidToEqual('true');

        DipList.expectAddButtonDisabled();
        DipList.expectLabel('Array of strings');
        DipList.expectDescription('Find the description here');

        for (const index of R.range(1, 5)) {
            DipList.expectSelectedItemToEqual(index, `Item ${index}`);
        }
    });

    it('should be able add and remove items', () => {
        DipList.removeSelectedItem(1);
        TestBindings.expectModelToEqual('["Item 2","Item 3","Item 4"]');
        DipList.setValue('n');
        DipList.expectClearSelectionPresent();
        DipList.clickAddButton();
        DipList.expectClearSelectionNotPresent();
        DipList.expectAddButtonDisabled();
        TestBindings.expectModelToEqual('["Item 2","Item 3","Item 4","n"]');
        DipList.expectSelectedItemToEqual(1, 'Item 2');
        DipList.expectSelectedItemToEqual(2, 'Item 3');
        DipList.expectSelectedItemToEqual(3, 'Item 4');
        DipList.expectSelectedItemToEqual(4, 'n');
    });

    it('should be disabled when "disabled" option is chosen', () => {
        DipList.expectAddButtonDisabled();
        DipList.expectClearSelectionNotPresent();
        DipList.expectNotReadonlyInput();
        DipList.removeClearSelectedItemPresent(1);
        DipList.setValue('n');
        DipList.expectAddButtonEnabled();
        DipList.expectClearSelectionPresent();
        DipList.clickOnLabel();

        TestBindings.toggleDisabled();
        DipList.expectAddButtonDisabled();
        DipList.removeClearSelectedItemNotPresent(1);
        DipList.expectReadonlyInput();

        TestBindings.toggleDisabled();
        DipList.expectAddButtonEnabled();
        DipList.expectNotReadonlyInput();
        DipList.expectClearSelectionPresent();
        DipList.removeClearSelectedItemPresent(1);
    });

    it('should behave properly in different display modes', () => {
        TestBindings.displayModeDisplay();
        DipList.expectInputNotPresent();
        DipList.expectAddButtonNotPresent();
        DipList.removeClearSelectedItemNotPresent(1);
        DipList.expectDescription('Find the description here');

        TestBindings.displayModeCreation();
        DipList.expectNotReadonlyInput();
        DipList.expectAddButtonEnabled();
        DipList.removeClearSelectedItemPresent(1);
        DipList.expectDescription('Find the description here');

        TestBindings.displayModeEdit();
        DipList.expectNotReadonlyInput();
        DipList.expectAddButtonEnabled();
        DipList.removeClearSelectedItemPresent(1);
        DipList.expectDescription('Find the description here');
    });
});
