import json from 'format-json';
import Navigation from '../../../dsl/common/navigation';
import TestBindings from '../../../dsl/common/test-bindings';
import DipMapStringString from '../../../dsl/component/dip/dip-map-string-string';

describe('XL Dip Map String String Encrypted Scenario', () => {
    beforeAll(() => {
        Navigation.openDipElement('dipMapStringString1');
        TestBindings.togglePassword();
    });

    afterAll(() => {
        TestBindings.togglePassword();
        TestBindings.resetPristinity();
        Navigation.openHomePage();
    });

    it('should have right initial state', () => {
        TestBindings.expectModelToEqual(json.plain({
            Name: 'Marc',
            Surname: 'Dupont',
            Age: 39,
            Street: '433 Boulevard des templiers',
            Zipcode: 74003,
            City: 'Paris',
            Phone: '0123456789',
            'Mobile Phone': '+31123456789'
        }));

        TestBindings.expectPristineToEqual('true');
        TestBindings.expectValidToEqual('true');
        DipMapStringString.expectDescription('Find the description here');
        DipMapStringString.expectLabel('Dictionary');
        DipMapStringString.expectFilterPresent();
        DipMapStringString.expectAddButtonEnabled();

        DipMapStringString.expectKeyCellValue(1, 'Age');
        DipMapStringString.expectValueCellBePasswordType(1);

        DipMapStringString.expectKeyCellValue(2, 'City');
        DipMapStringString.expectValueCellBePasswordType(2);

        DipMapStringString.expectKeyCellValue(3, 'Mobile Phone');
        DipMapStringString.expectValueCellBePasswordType(3);

        DipMapStringString.expectKeyCellValue(4, 'Name');
        DipMapStringString.expectValueCellBePasswordType(4);

        DipMapStringString.expectKeyCellValue(5, 'Phone');
        DipMapStringString.expectValueCellBePasswordType(5);

        DipMapStringString.clickAddButton(true);

        DipMapStringString.expectKeyCellValue(1, '');
        DipMapStringString.expectValueCellValue(1, '', true);
        DipMapStringString.expectKeyCellPlaceholder(1, 'Add key...');
        DipMapStringString.expectValueCellPlaceholder(1, 'Add value...', true);

        DipMapStringString.setKeyValue(1, 'a', true);
        DipMapStringString.setFilterValue('a');
        DipMapStringString.removeEntry(1);
    });

    it('should display icon to show password for new values', () => {
        DipMapStringString.expectAddButtonEnabled();
        DipMapStringString.clickAddButton(true);
        DipMapStringString.expectShowPasswordIconNotPresent(1);
        DipMapStringString.setEntry(1, 'secretKey', 'secretValue', true);
        DipMapStringString.expectAddButtonEnabled();
        DipMapStringString.expectShowPasswordIconPresent(1);
        DipMapStringString.expectValueCellBePasswordType(1);
        DipMapStringString.clickShowPasswordButton(1);
        DipMapStringString.expectValueCellBeTextType(1);
        DipMapStringString.clickUnshowPasswordButton(1);
        DipMapStringString.expectValueCellBePasswordType(1);
    });

    it('should display icon to show password for modified values', () => {
        DipMapStringString.setEntry(3, 'City', '', true);
        DipMapStringString.setEntry(3, 'City', 'asdf', true);
        DipMapStringString.expectShowPasswordIconPresent(3);
        DipMapStringString.expectValueCellBePasswordType(3);
        DipMapStringString.expectShowPasswordIconPresent(3);
        DipMapStringString.clickShowPasswordButton(3);
        DipMapStringString.expectValueCellBeTextType(3);
        DipMapStringString.clickUnshowPasswordButton(3);
        DipMapStringString.expectValueCellBePasswordType(3);
        DipMapStringString.clickShowPasswordButton(3);
        DipMapStringString.expectValueCellBeTextType(3);
        TestBindings.resetPristinity();
        DipMapStringString.expectShowPasswordIconNotPresent(3);
        DipMapStringString.expectValueCellBePasswordType(3);
    });

    it('should not be able to search by value', () => {
        DipMapStringString.setFilterValue('39');
        DipMapStringString.expectNoSearchResults();
    });
});
