import R from 'ramda';
import Navigation from '../../../dsl/common/navigation';
import TestBindings from '../../../dsl/common/test-bindings';
import DipSetOfStrings from '../../../dsl/component/dip/dip-set-of-strings';

describe('XL Dip Set Of Strings Scenario', () => {
    beforeAll(() => {
        Navigation.openDipElement('dipStringSet1');
    });

    afterAll(() => {
        TestBindings.resetPristinity();
    });

    it('should have right initial state', () => {
        TestBindings.expectModelToEqual('["Item 1","Item 2","Item 3","Item 4"]');
        TestBindings.expectPristineToEqual('true');
        TestBindings.expectValidToEqual('true');
        DipSetOfStrings.expectDescription('Find the description here');
        DipSetOfStrings.expectLabel('My items');
        DipSetOfStrings.expectFilterPresent();
        DipSetOfStrings.expectAddButtonEnabled();
        DipSetOfStrings.expectNoErrorMessage();

        DipSetOfStrings.expectKeyCellValue(1, 'Item 1');
        DipSetOfStrings.expectKeyCellValue(2, 'Item 2');
        DipSetOfStrings.expectKeyCellValue(3, 'Item 3');
        DipSetOfStrings.expectKeyCellValue(4, 'Item 4');
    });

    it('should auto focus key filed while adding a new row', () => {
        DipSetOfStrings.expectAddButtonEnabled();
        DipSetOfStrings.clickAddButton();
        DipSetOfStrings.expectKeyInputFocused();
    });

    it('should be able to sort asc/desc by click column header', () => {
        DipSetOfStrings.clickOnKeyHeader();

        DipSetOfStrings.expectKeyCellValue(1, 'Item 1');
        DipSetOfStrings.expectKeyCellValue(2, 'Item 2');
        DipSetOfStrings.expectKeyCellValue(3, 'Item 3');

        DipSetOfStrings.clickOnKeyHeader();

        DipSetOfStrings.expectKeyCellValue(1, 'Item 4');
        DipSetOfStrings.expectKeyCellValue(2, 'Item 3');
        DipSetOfStrings.expectKeyCellValue(3, 'Item 2');

        DipSetOfStrings.clickOnKeyHeader();
    });

    it('should show error if key is duplicated', () => {
        DipSetOfStrings.clickAddButton();
        DipSetOfStrings.setKeyValue(1, 'Item 2');
        DipSetOfStrings.expectDuplicatedKey('Item 2');
        DipSetOfStrings.removeEntry(1);
        DipSetOfStrings.expectNoErrorMessage();
    });

    it('should filter items', () => {
        DipSetOfStrings.expectFilterPresent();
        DipSetOfStrings.setFilterValue('4');
        DipSetOfStrings.expectKeyCellValue(1, 'Item 4');
        DipSetOfStrings.clearFilter();
        DipSetOfStrings.expectKeyCellValue(1, 'Item 1');
    });

    it('should filter when disabled', () => {
        TestBindings.toggleDisabled();
        DipSetOfStrings.setFilterValue('4');
        DipSetOfStrings.expectKeyCellValue(1, 'Item 4');
        DipSetOfStrings.clearFilter();
        TestBindings.toggleDisabled();
    });

    it('should filter when in visual display mode', () => {
        TestBindings.displayModeDisplay();
        DipSetOfStrings.setFilterValue('4');
        DipSetOfStrings.expectKeyCellValue(1, 'Item 4');
        DipSetOfStrings.clearFilter();
        TestBindings.displayModeEdit();
    });

    it('should be readonly when disabled or in visual display mode', () => {
        TestBindings.toggleDisabled();
        DipSetOfStrings.expectAddButtonPresent();
        DipSetOfStrings.expectRemoveEntryNotPresent(1);
        DipSetOfStrings.expectReadonlyKeyCell(1);

        TestBindings.displayModeDisplay();
        DipSetOfStrings.expectAddButtonPresent();
        DipSetOfStrings.expectRemoveEntryNotPresent(1);
        DipSetOfStrings.expectReadonlyKeyCell(1);

        TestBindings.toggleDisabled();
        DipSetOfStrings.expectAddButtonPresent();
        DipSetOfStrings.expectRemoveEntryNotPresent(1);
        DipSetOfStrings.expectReadonlyKeyCell(1);

        TestBindings.displayModeCreation();
    });

    it('should be able to remove items', () => {
        DipSetOfStrings.removeEntry(1);
        DipSetOfStrings.removeEntry(1);

        TestBindings.expectPristineToEqual('false');

        DipSetOfStrings.expectKeyCellValue(1, 'Item 3');
        DipSetOfStrings.expectKeyCellValue(2, 'Item 4');
    });

    it('should be able to add items', () => {
        DipSetOfStrings.expectAddButtonEnabled();
        DipSetOfStrings.clickAddButton();
        DipSetOfStrings.setKeyValue(1, 'An Item 2');
        DipSetOfStrings.clickOnFilter();
        DipSetOfStrings.expectKeyCellValue(1, 'An Item 2');
    });

    it('should show error message when no values and component is required to have a value', () => {
        TestBindings.toggleRequired();
        DipSetOfStrings.expectDescription('Find the description here');
        R.forEach(() => DipSetOfStrings.removeEntry(1), R.range(1, 4));
        DipSetOfStrings.expectError('Custom required text overriding default required text!');
        TestBindings.toggleRequired();
    });
});
