import Navigation from '../../dsl/common/navigation';
import Tab from '../../dsl/component/tab/tab';

describe('XL Tab System Scenario', () => {
    beforeAll(() => Navigation.openTabSystemPage());

    it('should properly display initial tabs', () => {
        Tab.expectActiveTabTitle('xl-react-widget-string');
        Tab.expectTabTitle(1, 'xl-react-widget-string');
        Tab.expectTabTitle(2, 'xl-react-widget-select-with-very-long-title');
    });

    it('should be possible to add and remove tabs', () => {
        Tab.closeTab(1);
        Tab.waitTabNotPresent(2);
        Tab.expectActiveTabTitle('xl-react-widget-select-with-very-long-title');
        Tab.addNewTab(2);
        Tab.expectTabTitle(1, 'xl-react-widget-select-with-very-long-title');
    });

    it('should be possible to toggle spinners in tabs', () => {
        Tab.expectNoSpinner();
        Tab.toggleLoadingState();
        Tab.expectSpinner();
        Tab.toggleLoadingState();
        Tab.expectNoSpinner();
    });

    it('should switch skins accordingly', () => {
        Tab.expectNoBlueSkin();
        Tab.expectDefaultSkin();
        Tab.expectNoNoneSkin();
        Tab.expectNoUnderlinedSkin();

        Tab.selectBlueSkin();
        Tab.expectBlueSkin();
        Tab.expectNoDefaultSkin();
        Tab.expectNoNoneSkin();
        Tab.expectNoUnderlinedSkin();

        Tab.selectDefaultSkin();
        Tab.expectNoBlueSkin();
        Tab.expectDefaultSkin();
        Tab.expectNoNoneSkin();
        Tab.expectNoUnderlinedSkin();

        Tab.selectNoneSkin();
        Tab.expectNoBlueSkin();
        Tab.expectNoDefaultSkin();
        Tab.expectNoneSkin();
        Tab.expectNoUnderlinedSkin();

        Tab.selectUnderlinedSkin();
        Tab.expectNoBlueSkin();
        Tab.expectNoDefaultSkin();
        Tab.expectNoNoneSkin();
        Tab.expectUnderlinedSkin();
    });
});
