import R from 'ramda';
import Navigation from '../../../dsl/common/navigation';
import TestBindings from '../../../dsl/common/test-bindings';
import WidgetAutocomplete from '../../../dsl/component/widget/widget-autocomplete';

describe('XL Widget Autocomplete Model Objects Scenario', () => {
    afterAll(() => TestBindings.resetPristinity());
    beforeAll(() => Navigation.openWidgetElement('widgetAutocomplete3'));

    it('should have right initial state show all suggestions when pristine', () => {
        TestBindings.expectPristineToEqual('true');
        TestBindings.expectObjectModelToEqual({
            id: 'Application2',
            type: 2
        });
        WidgetAutocomplete.expectLabel('Autocomplete*');
        WidgetAutocomplete.expectDescription('This is my autocomplete field.');
        WidgetAutocomplete.openSelection();
        for (const optionNumber of R.range(1, 7)) {
            WidgetAutocomplete.expectSuggestionValue(optionNumber, `Application${optionNumber}`);
        }
        WidgetAutocomplete.clearSelection();
    });

    it('should display only filtered items and be able to select options', () => {
        WidgetAutocomplete.setValue('1');
        WidgetAutocomplete.expectSuggestionValue(1, 'Application1');
        WidgetAutocomplete.selectOption(1);
        WidgetAutocomplete.expectSuggestionListNotPresent();
        TestBindings.expectValidToEqual('true');
        TestBindings.expectObjectModelToEqual({
            id: 'Application1',
            type: 1
        });
        WidgetAutocomplete.openSelection();
        for (const optionNumber of R.range(1, 7)) {
            WidgetAutocomplete.expectSuggestionValue(optionNumber, `Application${optionNumber}`);
        }
    });

    it('should show error if remove the model', () => {
        WidgetAutocomplete.clearSelection();
        TestBindings.expectObjectModelToEqual({});
        WidgetAutocomplete.expectError('Custom required error message overrides!');
    });

    it('should display a suggestion list', () => {
        WidgetAutocomplete.clickOnInput();
        WidgetAutocomplete.expectSuggestionListIsPresent();
        WidgetAutocomplete.clickOnLabel();
        WidgetAutocomplete.expectSuggestionListNotPresent();
    });
});
