import Navigation from '../../../dsl/common/navigation';
import TestBindings from '../../../dsl/common/test-bindings';
import WidgetString from '../../../dsl/component/widget/widget-string';

describe('XL Widget String Password Scenario', () => {
    beforeAll(() => {
        Navigation.openWidgetElement('widgetString1');
        TestBindings.setInputType('password');
    });

    afterAll(() => {
        TestBindings.setInputType('text');
        TestBindings.resetPristinity();
    });

    it('should have right initial state', () => {
        TestBindings.expectModelToEqual('');
        TestBindings.expectValidToEqual('true');
        TestBindings.expectPristineToEqual('true');
        WidgetString.expectPlaceholderToEqual('String placeholder');
        WidgetString.expectDescription('This is my string field.');
        WidgetString.expectLabel('String me');
        WidgetString.expectInputTypeToEqual('password');
    });

    it('should be disabled when "disabled" option is chosen', () => {
        WidgetString.expectNotReadonlyInput();
        TestBindings.toggleDisabled();
        WidgetString.expectReadonlyInput();
        TestBindings.toggleDisabled();
        WidgetString.expectNotReadonlyInput();
    });

    it('should behave properly in different display modes', () => {
        TestBindings.displayModeDisplay();
        WidgetString.expectReadonlyInput();
        TestBindings.displayModeCreation();
        WidgetString.expectNotReadonlyInput();
        TestBindings.displayModeEdit();
        WidgetString.expectNotReadonlyInput();
    });

    it('should display cross if any text is typed in input field', () => {
        WidgetString.setValue('1');
        TestBindings.expectModelToEqual(1);
        WidgetString.expectClearIconPresent();
        TestBindings.expectPristineToEqual('false');
        WidgetString.clearValue();
        TestBindings.expectPristineToEqual('false');
        WidgetString.expectClearIconNotPresent();
        TestBindings.expectModelToEqual('');
        TestBindings.resetPristinity();
    });

    it('should not display an icon for showing passwords if the field is pristine', () => {
        WidgetString.setValue('1');
        TestBindings.expectModelToEqual(1);
        WidgetString.expectShowPasswordIconPresent();
        WidgetString.clearValue();
        TestBindings.expectModelToEqual('');
    });

    it('should not display an icon for showing passwords if the field is empty', () => {
        TestBindings.expectModelToEqual('');
        WidgetString.expectShowPasswordIconNotPresent();
    });

    it('should only display an icon for showing passwords if the field is type password', () => {
        TestBindings.setInputType('text');
        WidgetString.setValue('1');
        TestBindings.expectModelToEqual(1);
        WidgetString.expectShowPasswordIconNotPresent();
        WidgetString.clearValue();
        TestBindings.expectModelToEqual('');
        TestBindings.setInputType('password');
    });

    it('should display an icon for showing passwords only on password fields when not pristine and have value', () => {
        WidgetString.expectShowPasswordIconNotPresent();
        WidgetString.setValue('1');
        TestBindings.expectModelToEqual(1);
        TestBindings.expectPristineToEqual('false');
        WidgetString.expectShowPasswordIconPresent();
        WidgetString.clearValue();
        TestBindings.expectModelToEqual('');
    });

    it('should toggle visibility of passwords when clicking the button', () => {
        WidgetString.setValue('1');
        TestBindings.expectModelToEqual(1);
        WidgetString.expectInputTypeToEqual('password');
        WidgetString.clickShowPasswordButton();
        WidgetString.expectInputTypeToEqual('text');
        WidgetString.clickUnshowPasswordButton();
        WidgetString.expectInputTypeToEqual('password');
        WidgetString.clearValue();
        TestBindings.expectModelToEqual('');
    });
});
