import Navigation from '../../../dsl/common/navigation';
import WidgetTable from '../../../dsl/component/widget/widget-table';

describe('XL Widget Table Scenario', () => {
    beforeAll(() => Navigation.openWidgetElement('widgetTable'));

    it('should contain first name, last name, city and state headers', () => {
        WidgetTable.expectHeaders(['Firstname', 'Address', 'City', 'State']);
    });

    it('should be able to go through the pagination pages', () => {
        WidgetTable.openPage(2);
        WidgetTable.expectActivePage(2);
        WidgetTable.openPage(1);
        WidgetTable.expectActivePage(1);
    });

    it('should be able to change pagination size', () => {
        WidgetTable.changePaginationSize(5);
        WidgetTable.expectRowPresent(5);
        WidgetTable.changePaginationSize(2);
    });

    it('should be able to toggle filters', () => {
        WidgetTable.toggleFilters();
        WidgetTable.expectFiltersPresent();
        WidgetTable.toggleFilters();
    });

    it('should be able to filter data by one column', () => {
        WidgetTable.toggleFilters();
        WidgetTable.setFilter(1, 'joan');
        WidgetTable.expectFirstName(1, 'Joan B. Jones');
        WidgetTable.expectRowNotPresent(2);
        WidgetTable.toggleFilters();
    });

    it('should be able to filter data by multiple columns', () => {
        WidgetTable.toggleFilters();
        WidgetTable.setFilter(1, 'bob');
        WidgetTable.setFilter(2, '9996');
        WidgetTable.expectFirstName(1, 'Bob C. Uncle');
        WidgetTable.expectRowNotPresent(2);
        WidgetTable.toggleFilters();
    });

    it('should be able to clear out the filter by column', () => {
        WidgetTable.toggleFilters();
        WidgetTable.setFilter(1, 'joan');
        WidgetTable.resetFilter(1);
        WidgetTable.expectFirstName(1, 'John A. Smith');
        WidgetTable.expectFirstName(2, 'Joan B. Jones');
        WidgetTable.toggleFilters();
    });

    it('should be able to clear out all the filters', () => {
        WidgetTable.toggleFilters();
        WidgetTable.setFilter(1, 'bob');
        WidgetTable.setFilter(2, '9996');
        WidgetTable.resetAllFilters();
        WidgetTable.expectFirstName(1, 'John A. Smith');
        WidgetTable.expectFirstName(2, 'Joan B. Jones');
        WidgetTable.toggleFilters();
    });

    it('should be able to sort by firstname', () => {
        WidgetTable.sortColumn(1);
        WidgetTable.expectDescOrder(1);
        WidgetTable.expectFirstName(1, 'John Doe');
        WidgetTable.expectFirstName(2, 'John D. Smith');

        WidgetTable.sortColumn(1);
        WidgetTable.expectAscOrder(1);
        WidgetTable.expectFirstName(1, 'Bob C. Uncle');
        WidgetTable.expectFirstName(2, 'Bob F. Uncle');
    });

    describe('multi select functionality', () => {
        beforeAll(() => {
            WidgetTable.toggleMultiSelectColumns();
            WidgetTable.openPage(2);
            WidgetTable.openPage(1);
            WidgetTable.changePaginationSize(10);
        });

        afterAll(() => {
            WidgetTable.toggleMultiSelectColumns();
            WidgetTable.changePaginationSize(2);
            WidgetTable.openPage(2);
            WidgetTable.openPage(1);
        });

        it('should be able to select and deselect all rows', () => {
            WidgetTable.selectAllRows();
            WidgetTable.expectSelected([1, 2, 3, 4, 5, 6, 7, 8, 9, 10]);
            WidgetTable.selectAllRows();
            WidgetTable.expectSelected([]);
        });

        it('should be able to deselect some rows', () => {
            WidgetTable.selectRow(1);
            WidgetTable.selectRow(5);
            WidgetTable.selectRow(7);
            WidgetTable.expectSelected([1, 5, 7]);
            WidgetTable.selectAllRows();
            WidgetTable.expectSelected([]);
        });

        it('should be able to select and unselect some rows', () => {
            WidgetTable.selectRow(2);
            WidgetTable.selectRow(5);
            WidgetTable.selectRow(8);
            WidgetTable.expectSelected([2, 5, 8]);
            WidgetTable.selectRow(2);
            WidgetTable.selectRow(5);
            WidgetTable.selectRow(8);
            WidgetTable.expectSelected([]);
        });
    });
});
