import {
    getBaseName,
    ERROR,
    hasInputHints,
    isError,
    isWarning,
    WARNING,
    withoutRootNode,
    getInputHintValues
} from '~/app/utils/ci-utils';

describe('ci-utils', () => {
    describe('withoutRootNode', () => {
        it('should return CI id as is', () => {
            expect(withoutRootNode()).toEqual();
            expect(withoutRootNode(false)).toEqual(false);
            expect(withoutRootNode('')).toEqual('');
        });
        it('should remove the root of CI id', () => {
            expect(withoutRootNode('Applications')).toEqual('Applications');
            expect(withoutRootNode('Applications/TestApp/1.0')).toEqual('TestApp/1.0');
            expect(withoutRootNode('Environments/MyDir/Staging/DepApp-1.0')).toEqual('MyDir/Staging/DepApp-1.0');
        });
    });

    describe('getBaseName', () => {
        it('should extract base name', () => {
            expect(getBaseName('Applications/TestApp')).toEqual('TestApp');
            expect(getBaseName('Applications/TestApp/1.0')).toEqual('1.0');
        });
    });

    describe('isError', () => {
        it('should check for errors', () => {
            expect(isError(false, {})).toEqual(false);
            expect(isError(false, {show: false, type: ERROR})).toEqual(false);
            expect(isError(false, {show: true, type: ERROR})).toEqual(true);
            expect(isError(false, {show: true, type: WARNING})).toEqual(false);

            expect(isError(true, {})).toEqual(false);
            expect(isError(true, {show: false, type: ERROR})).toEqual(false);
            expect(isError(true, {show: true, type: ERROR})).toEqual(false);
            expect(isError(true, {show: true, type: WARNING})).toEqual(false);
        });

        it('should check for warnings', () => {
            expect(isWarning(false, {})).toEqual(false);
            expect(isWarning(false, {show: false, type: ERROR})).toEqual(false);
            expect(isWarning(false, {show: true, type: ERROR})).toEqual(false);
            expect(isWarning(false, {show: false, type: WARNING})).toEqual(false);
            expect(isWarning(false, {show: true, type: WARNING})).toEqual(false);

            expect(isWarning(true, {})).toEqual(false);
            expect(isWarning(true, {show: false, type: ERROR})).toEqual(false);
            expect(isWarning(true, {show: true, type: ERROR})).toEqual(false);
            expect(isWarning(true, {show: true, type: WARNING})).toEqual(true);
        });
    });

    describe('hasInputHints', () => {
        it('should check for inputHint', () => {
            expect(hasInputHints({})).toEqual(false);
            expect(hasInputHints({inputHint: {}})).toEqual(false);
            expect(hasInputHints({inputHint: {values: []}})).toEqual(false);
            expect(hasInputHints({inputHint: {values: [{label: 'hint', value: 'value'}]}})).toEqual(true);
        });
    });

    describe('getInputHintValues', () => {
        it('should get inputHint values', () => {
            expect(getInputHintValues({})).toEqual(undefined);
            expect(getInputHintValues({inputHint: {}})).toEqual(undefined);
            expect(getInputHintValues({inputHint: {values: []}})).toEqual([]);
            expect(getInputHintValues(
                {inputHint: {values: [{label: 'hint', value: 'value'}]}})
            ).toEqual([{label: 'hint', value: 'value'}]);
        });
    });
});
