import moment from 'moment';
import {isDateInRange} from '~/app/utils/date-utils';

describe('date-utils', () => {
    describe('isDateInRange', () => {
        it('should be invalid if some parameter is not defined', () => {
            const checkedDate = moment('20110101', 'YYYYMMDD');
            const maxDate = moment('20130101', 'YYYYMMDD');
            const minDate = moment('20120101', 'YYYYMMDD');
            expect(isDateInRange({maxDate, minDate})).toBeFalsy();
            expect(isDateInRange({checkedDate, minDate})).toBeFalsy();
        });
        it('should be invalid on date which less than minimum date', () => {
            const checkedDate = moment('20110101', 'YYYYMMDD');
            const maxDate = moment('20130101', 'YYYYMMDD');
            const minDate = moment('20120101', 'YYYYMMDD');
            expect(isDateInRange({checkedDate, maxDate, minDate})).toBeFalsy();
        });
        it('should be invalid on date which more than maximum date', () => {
            const checkedDate = moment('20140101', 'YYYYMMDD');
            const maxDate = moment('20130101', 'YYYYMMDD');
            const minDate = moment('20120101', 'YYYYMMDD');
            expect(isDateInRange({checkedDate, maxDate, minDate})).toBeFalsy();
        });
        it('should ignore minimum date if it is not specified', () => {
            const checkedDate = moment('20100101', 'YYYYMMDD');
            const maxDate = moment('20130101', 'YYYYMMDD');
            expect(isDateInRange({checkedDate, maxDate})).toBeTruthy();
        });
        it('should ignore maximum date if it is not specified', () => {
            const checkedDate = moment('20200101', 'YYYYMMDD');
            const minDate = moment('20110101', 'YYYYMMDD');
            expect(isDateInRange({checkedDate, minDate})).toBeTruthy();
        });
        it('should ignore maximum and minimum dates if they are not specified', () => {
            const checkedDate = moment('20200101', 'YYYYMMDD');
            expect(isDateInRange({checkedDate})).toBeTruthy();
        });
        it('should be valid if checked date in between minimum and maximum dates', () => {
            const checkedDate = moment('20120101', 'YYYYMMDD');
            const maxDate = moment('20130101', 'YYYYMMDD');
            const minDate = moment('20120101', 'YYYYMMDD');
            expect(isDateInRange({checkedDate, maxDate, minDate})).toBeTruthy();
        });
        it('should be valid if checked date is same as minimum date independent of time', () => {
            const minDate = moment();
            const maxDate = minDate.add(1, 'month');
            const checkedDate = minDate.add(1, 'minute');
            expect(isDateInRange({checkedDate, maxDate, minDate})).toBeTruthy();
        });
    });
});
