import {findFirstFieldNameWithFalseValue, transformItem, transformModel} from '~/app/utils/transform-utils';

describe('TransformUtils', () => {
    describe('findFirstFieldNameWithFalseValue', () => {
        it('should find the name of the first field which has false value', () => {
            expect(findFirstFieldNameWithFalseValue({a1: true, a2: false, a3: false, a4: true})).toEqual('a2');
        });
        it('should return undefined if nothing had been found', () => {
            expect(findFirstFieldNameWithFalseValue({a1: true, a2: true, a3: true, a4: true})).toBeUndefined();
            expect(findFirstFieldNameWithFalseValue({})).toBeUndefined();
        });
    });

    describe('transformItem', () => {
        it('should transform object and map it by specified objLabel', () => {
            expect(transformItem({id: 'AppId', ref: 'AppRef'}, 'id')).toEqual('AppId');
        });

        it('should not transform value if it is not an object', () => {
            expect(transformItem('AppId', undefined)).toEqual('AppId');
        });
    });

    describe('transformModel', () => {
        it('should transform object and map it by specified objLabel', () => {
            expect(transformModel('id')([{id: 'AppId1'}, {id: 'AppId2'}])).toEqual(['AppId1', 'AppId2']);
        });
    });
});
