import {apiFocus} from '~/app/xl-react-dip/xl-react-dip-lookup/xl-react-dip-lookup-service';

describe('xl-react-dip-lookup-service', () => {
    describe('apiFocus', () => {
        const field1Focus = jest.fn();
        const field2Focus = jest.fn();
        const field3Focus = jest.fn();

        afterEach(() => {
            jest.clearAllMocks();
        });

        const baseContext = {
            componentApis: {
                field1: {focus: field1Focus},
                field2: {focus: field2Focus},
                field3: {focus: field3Focus}
            }
        };

        it('should focus first invalid field', () => {
            const context = {
                ...baseContext,
                valid: {
                    field1: true,
                    field2: false,
                    field3: true
                }
            };

            apiFocus(context);

            expect(field1Focus).not.toBeCalled();
            expect(field2Focus).toBeCalled();
            expect(field3Focus).not.toBeCalled();
        });

        it('should focus first field if all fields are valid', () => {
            const context = {
                ...baseContext,
                valid: {
                    field1: true,
                    field2: true,
                    field3: true
                }
            };

            apiFocus(context);

            expect(field1Focus).toBeCalled();
            expect(field2Focus).not.toBeCalled();
            expect(field3Focus).not.toBeCalled();
        });

        it('should focus first field if field validity is unknown - nil case', () => {
            const context = {...baseContext};

            apiFocus(context);

            expect(field1Focus).toBeCalled();
            expect(field2Focus).not.toBeCalled();
            expect(field3Focus).not.toBeCalled();
        });

        it('should focus first field if field validity is unknown - empty case', () => {
            const context = {
                ...baseContext,
                valid: {}
            };

            apiFocus(context);

            expect(field1Focus).toBeCalled();
            expect(field2Focus).not.toBeCalled();
            expect(field3Focus).not.toBeCalled();
        });
    });
});
