import {
    externalModelToInternal,
    getExternalPropertyKeys,
    internalModelToExternal,
    transformModelOnToggle
} from '~/app/xl-react-form/form-model-utils';

describe('form-model-utils', () => {
    describe('externalModelToInternal', () => {
        it('should transform external model to internal representation model of the form', () => {
            expect(externalModelToInternal({
                'external-properties': {
                    vault: {
                        key: 'SECRET',
                        kind: 'lookup',
                        provider: 'Configurations/Vault/LookupValueProvider'
                    }
                },
                'password': 'secret'
            })).toEqual({
                password: 'secret',
                vault: {
                    key: 'SECRET',
                    kind: 'lookup',
                    provider: 'Configurations/Vault/LookupValueProvider'
                }
            });
        });
    });

    describe('internalModelToExternal', () => {
        it('should transform internal model to external usage', () => {
            expect(internalModelToExternal({
                password: 'secret',
                vault: {
                    key: 'SECRET',
                    kind: 'lookup',
                    provider: 'Configurations/Vault/LookupValueProvider'
                }
            }, ['vault'])).toEqual({
                'external-properties': {
                    vault: {
                        key: 'SECRET',
                        kind: 'lookup',
                        provider: 'Configurations/Vault/LookupValueProvider'
                    }
                },
                'password': 'secret'
            });
        });
    });

    describe('getExternalPropertyKeys', () => {
        it('should return the list of keys for all external properties', () => {
            expect(getExternalPropertyKeys({
                'external-properties': {
                    vault: {
                        key: 'SECRET',
                        kind: 'lookup',
                        provider: 'Configurations/Vault/LookupValueProvider'
                    }
                },
                'password': 'secret'
            })).toEqual(['vault']);
        });
    });

    describe('transformModelOnToggle', () => {
        it('should transform model from lookup to string value', () => {
            expect(transformModelOnToggle({
                'external-properties': {
                    vault: {
                        key: 'SECRET',
                        kind: 'lookup',
                        provider: 'Configurations/Vault/LookupValueProvider'
                    }
                },
                'password': 'secret'
            }, 'vault', false)).toEqual({
                password: 'secret',
                vault: undefined
            });
        });

        it('should transform model from string value to lookup model', () => {
            expect(transformModelOnToggle({
                password: 'secret',
                vault: 'my-vault-secret'
            }, 'vault', true)).toEqual({
                'external-properties': {
                    vault: {
                        key: '',
                        kind: 'lookup',
                        provider: ''
                    }
                },
                'password': 'secret'
            });
        });
    });
});
