import R from 'ramda';
import {ERROR} from '../../../app/utils/ci-utils';
import {createCiPropertyValidators} from '~/app/xl-react-form/validator-utils';

describe('validator-utils', () => {
    describe('createCiPropertyValidators', () => {
        const ciProperty = {
            description: 'Description component String',
            inputHint: {
                rules: [
                    {
                        message: 'Value \'%s\' should start with comp',
                        pattern: '^comp'
                    },
                    {
                        message: 'Value \'%s\' should end on 0',
                        pattern: '0$'
                    }
                ]
            },
            kind: 'STRING',
            label: 'Label component String',
            name: 'nameComponentString',
            password: false,
            required: false,
            size: 'DEFAULT',
            transient: false
        };

        const ciPropertyWithRequiredInputHint = {
            ...ciProperty,
            inputHint: {required: true}
        };

        const validateModel = (validators, model) => {
            const evaluatedValidations = R.map((validator) => validator(model), validators);
            const validationToShow = R.find((result) => result.condition, evaluatedValidations);
            return R.path(['message', 'text'], validationToShow);
        };

        const defaultValidator = () => (_, model) => ({
            condition: R.isNil(model) || R.isEmpty(model),
            message: {show: true, text: 'This field is required', type: ERROR}
        });

        it('should trigger a rule only for defined value', () => {
            const validators = createCiPropertyValidators({validators: []}, ciProperty);
            expect(validateModel(validators, '')).toEqual('Value \'\' should start with comp');
        });

        it('should create validators for warnings and errors for a specific CI property', () => {
            const validators = createCiPropertyValidators({validators: [defaultValidator()]}, ciProperty);

            expect(validateModel(validators, 'com')).toEqual('Value \'com\' should start with comp');
            expect(validateModel(validators, 'comp')).toEqual('Value \'comp\' should end on 0');
            expect(validateModel(validators, 'comp0')).toBeUndefined();
            expect(validateModel(validators, '')).toEqual('This field is required');
        });

        it('should create required validators from provided metadata message hints', () => {
            const validators = createCiPropertyValidators({validators: []}, ciPropertyWithRequiredInputHint);
            expect(validateModel(validators, '')).toEqual('Value is required');
        });
    });
});
