import {testSaga} from 'redux-saga-test-plan';
import {call} from 'redux-saga/effects';
import {filterSaga} from '~/app/xl-react-widget/xl-react-widget-redux-table/sagas/xl-react-widget-redux-table-filter-saga'; // eslint-disable-line max-len
import {getTableById} from '~/app/xl-react-widget/xl-react-widget-redux-table/xl-react-widget-table-selectors';
import tableActions
    from '~/app/xl-react-widget/xl-react-widget-redux-table/xl-react-widget-redux-table-actions';
import {
    fetchData,
    withDelay,
    withSpinner
} from '~/app/xl-react-widget/xl-react-widget-redux-table/sagas/xl-react-widget-redux-table-common-saga';
import {buildFilterQuery} from '~/app/xl-react-widget/xl-react-widget-redux-table/query-builder-utils';

describe('Widget Redux Table Filter Saga', () => {
    const mockApi = jest.fn();
    const tableId = 'table-1';
    const filter = {};
    const prevFilter = [{
        fieldPath: 'afterDateOfBirth',
        model: ''
    }];

    describe('filterSaga', () => {
        it('should set filter query and fetch data based on this', () => {
            const api = mockApi;
            const filterDebounceTime = 100;
            const saga = testSaga(filterSaga, {api, filter, tableId});
            saga.next().select(getTableById, tableId);
            saga.next({filterDebounceTime, filters: prevFilter}).put(tableActions.setFilters(tableId, filter));
            const effect = call(withSpinner, tableId,
                call(fetchData, tableId, api, {change: {previous: buildFilterQuery(prevFilter), type: 'filter'}})
            );
            saga.next().call(withDelay, effect, filterDebounceTime);
            saga.next().isDone();
        });
    });
});
