import React from 'react';
import {mount} from 'enzyme';
import {XlReactWidgetCollapsibleVerticalSplitPanel} from '~/app/xl-react-widget/xl-react-widget-split-panel/xl-react-widget-collapsible-vertical-split-pane/xl-react-widget-collapsible-vertical-split-pane-component'; // eslint-disable-line max-len

import {XlReactWidgetCollapsibleVerticalSplitPanelConstants} from '~/app/xl-react-widget/xl-react-widget-split-panel/xl-react-widget-collapsible-vertical-split-pane/xl-react-widget-collapsible-vertical-split-pane-constants'; // eslint-disable-line max-len

const {SIDES} = XlReactWidgetCollapsibleVerticalSplitPanelConstants;

describe('XlReactWidgetCollapsibleVerticalSplitPanel', () => {
    it('should render correct', () => {
        const defaultProps = {
            isCollapsed: false,
            minPrimarySideWidth: 250,
            minSecondarySideWidth: 400,
            primarySideWidth: 300,
            sideToCollapse: SIDES.primary
        };
        const wrapper = mount(
            <XlReactWidgetCollapsibleVerticalSplitPanel {...defaultProps}>
                <h1>child1</h1>
                <h1>child2</h1>
            </XlReactWidgetCollapsibleVerticalSplitPanel>
        );
        const Pane1 = wrapper.find('.SplitPane > .Pane1');
        const Pane2 = wrapper.find('.SplitPane > .Pane2');
        expect(Pane1.length).toBe(1);
        expect(Pane2.length).toBe(1);
        expect(Pane1.props().size).toBe(300);
    });

    it('sideToCollapse: LEFT, should collapse left side when recieving updates', () => {
        const defaultProps = {
            isCollapsed: false,
            minPrimarySideWidth: 250,
            minSecondarySideWidth: 400,
            primarySideWidth: 300,
            sideToCollapse: SIDES.primary
        };
        const wrapper = mount(
            <XlReactWidgetCollapsibleVerticalSplitPanel {...defaultProps}>
                <h1>child1</h1>
                <h1>child2</h1>
            </XlReactWidgetCollapsibleVerticalSplitPanel>
        );
        const Pane1 = wrapper.find('.SplitPane > .Pane1');
        expect(Pane1.props().size).toBe(300);

        wrapper.setProps({isCollapsed: true});
        const Pane1Updated = wrapper.find('.SplitPane > .Pane1');
        expect(Pane1Updated.props().size).toBe('0%');
    });

    it('sideToCollapse: RIGHT, should collapse right side when recieving updates', () => {
        const defaultProps = {
            isCollapsed: false,
            minPrimarySideWidth: 250,
            minSecondarySideWidth: 400,
            primarySideWidth: 300,
            sideToCollapse: SIDES.secondary
        };
        const wrapper = mount(
            <XlReactWidgetCollapsibleVerticalSplitPanel {...defaultProps}>
                <h1>child1</h1>
                <h1>child2</h1>
            </XlReactWidgetCollapsibleVerticalSplitPanel>
        );
        const Pane1 = wrapper.find('.SplitPane > .Pane1');
        expect(Pane1.props().size).toBe(300);

        wrapper.setProps({isCollapsed: true});
        const Pane1Updated = wrapper.find('.SplitPane > .Pane1');
        expect(Pane1Updated.props().size).toBe('100%');
    });
});
