import React from 'react';
import {mount} from 'enzyme';
import timekeeper from 'timekeeper';
import {XlReactWidgetTime} from '~/app/xl-react-widget/xl-react-widget-time/xl-react-widget-time';

const now = new Date();
timekeeper.freeze(now);

describe('XlReactWidgetTime', () => {
    const onModelChange = jest.fn();

    const defaultProps = {
        metadata: {label: 'Time'},
        onModelChange
    };
    let wrapper;

    beforeEach(() => {
        wrapper = mount(<XlReactWidgetTime {...defaultProps}/>);
    });

    it('should render correctly', () => {
        const overlayTrigger = wrapper.find('OverlayTrigger input');
        const timeInput = overlayTrigger.find('[type="text"]');
        expect(timeInput).toHaveProp('disabled', false);
        expect(timeInput).toHaveProp('value');
        expect(wrapper.find('XlReactWidgetLabel')).toBeDefined();
        expect(wrapper.find('.xl-react-widget-time')).toBeDefined();
        expect(wrapper).toHaveState('pristine', true);
    });

    it('should disable the time input when property is disabled', () => {
        wrapper.setProps({disabled: true});
        const overlayTrigger = wrapper.find('OverlayTrigger');
        const timeInput = wrapper.find('[type="text"]');
        expect(overlayTrigger.length).toEqual(0);
        expect(timeInput).toHaveProp('disabled', true);
    });

    it('should be able to update the time through props', () => {
        const date = new Date();
        date.setHours(10);
        date.setMinutes(20);
        const wrapper2 = mount(<XlReactWidgetTime {...defaultProps} model={date}/>);
        const timeInput = wrapper2.find('OverlayTrigger').find('[type="text"]');
        expect(timeInput.prop('value')).toBe('10:20 AM');
    });
});
